import logging
from functools import reduce
from math import ceil

import dask
import numpy as np

from cidan.GUI.Data_Interaction.data_handler_functions.change_param import *
from cidan.GUI.Data_Interaction.data_handler_functions.export import \
    export_overlapping_rois, save_image, export, calculate_statistics
from cidan.GUI.Data_Interaction.data_handler_functions.gen_rois_functions import \
    gen_roi_display_variables, genRoiFromPoint, calculate_roi_extraction, \
    gen_class_display_variables
from cidan.GUI.Data_Interaction.data_handler_functions.load_filter_functions import \
    calculate_dataset, transform_data_to_zarr, reset_data, load_dataset, \
    compute_trial_filter_step, load_mask
from cidan.GUI.Data_Interaction.data_handler_functions.save_functions import \
    load_param_json, save_new_param_json
from cidan.GUI.Data_Interaction.data_handler_functions.time_trace_functions import \
    calculate_time_traces, get_time_trace
from cidan.LSSC.SpatialBox import SpatialBox
from cidan.LSSC.functions.data_manipulation import auto_crop as auto_crop_func
from cidan.LSSC.functions.eigen import loadEigenVectors
from cidan.LSSC.functions.pickle_funcs import *
from cidan.TimeTrace.deltaFOverF import calculateDeltaFOverF
from cidan.TimeTrace.mean import calculateMeanTrace, neuropil

logger1 = logging.getLogger("cidan.DataHandler")


def connected_components_graph(A_csr, param, return_labels):
    pass

# NOW rewrite the original initiation function to include new stuff
class DataHandler:
    """
    Interacts with the algorithm and stores the current image data

    This class is initialized whenever data is loaded into the program, it handles
    saving the parameter file, running each of the algorithms in timetrace and LSSC.

    Attributes
    ----------
    global_params : Dict
        This saves the general params of for the dataset, basically whether parameters
        have changed so that the dataset, filters, boxes, eigen vectors, or
        roi extraction need to be recalculated


    """
    # x is for roi data, y is for neuropil data, z is roi data denoised
    time_trace_possibilities_functions = {
        "Mean Florescence Denoised": lambda x, y: calculateMeanTrace(x, y),
        "Mean Florescence": lambda x, y: calculateMeanTrace(x, y),
        "DeltaF Over F Denoised": lambda x, y: calculateDeltaFOverF(x, y),
        "DeltaF Over F": lambda x, y: calculateDeltaFOverF(x, y),
        "Mean Floresence Denoised (Neuropil Corrected)": lambda x,
                                                                y: calculateMeanTrace(x,
                                                                                      y,
                                                                                      sub_neuropil=True),
        "Mean Floresence  (Neuropil Corrected)": lambda x, y: calculateMeanTrace(x,
                                                                                 y,

                                                                                 sub_neuropil=True),
        "DeltaF Over F  (Neuropil Corrected)": lambda x, y: calculateDeltaFOverF(x,
                                                                                 y,

                                                                                 sub_neuropil=True),
        "DeltaF Over F Denoised (Neuropil Corrected)": lambda x,
                                                              y: calculateDeltaFOverF(x,
                                                                                      y,

                                                                                      sub_neuropil=True),
        "Neuropil": lambda x, y: neuropil(x, y)
    }
    _color_list = [(218, 67, 34),
                   (132, 249, 22), (22, 249, 140), (22, 245, 249),
                   (22, 132, 249), (224, 22, 249), (249, 22, 160)]
    _base_class_construct = {"Unassigned": {"color": (150, 150, 150), "rois": [],
                                            "name": "Unassigned", "editable": False},
                             "Base": {"color": _color_list[0], "rois": [],
                                      "name": "Base", "editable": True},
                             "Important": {"color": _color_list[1], "rois": [],
                                           "name": "important", "editable": True}}
    _global_params_default = {
        "save_intermediate_steps": True,
        "need_recalc_dataset_params": True,
        "need_recalc_filter_params": True,
        "need_recalc_box_params": True,
        "need_recalc_eigen_params": True,
        "need_recalc_roi_extraction_params": True,
        "load_into_mem": True,
    }

    _dataset_params_default = {
        "dataset_folder_path": "",
        "trials_loaded": [],
        "trials_all": [],
        "mask_path": "",
        "single_file_mode": False,
        "original_folder_trial_split": "",
        "slice_stack": False,
        "slice_every": 3,
        "slice_start": 0,
        "crop_stack": False,
        "crop_x": [0, 0],
        "crop_y": [0, 0],
        "trial_split": False,
        "trial_length": 400,
        "auto_crop": False
    }

    _filter_params_default = {
        "median_filter": False,
        "median_filter_size": 3,
        "z_score": False,
        "hist_eq": False,
        "localSpatialDenoising": True,
        "pca": False,
        "pca_threshold": .97

    }
    _filter_params_default_widefield = {
        "median_filter": False,
        "median_filter_size": 3,
        "z_score": False,
        "hist_eq": False,
        "localSpatialDenoising": True,
        "pca": False,
        "pca_threshold": .97

    }
    _box_params_default = {
        "total_num_time_steps": 1,
        "total_num_spatial_boxes": 1,
        "spatial_overlap": 40
    }
    _eigen_params_default = {
        "eigen_vectors_already_generated": False,
        "num_eig": 50,
        "normalize_w_k": 32,
        "metric": "l2",
        "knn": 50,
        "accuracy": 75,
        "eigen_accuracy": 8,
        "connections": 40

    }
    _roi_extraction_params_default = {
        "elbow_threshold_method": True,
        "elbow_threshold_value": .95,
        "eigen_threshold_method": True,
        "eigen_threshold_value": .9,
        "num_eigen_vector_select": 1,
        "merge_temporal_coef": .99,
        "roi_size_min": 30,
        "roi_size_max": 2000,
        "merge": True,
        "num_rois": 60,
        "fill_holes": True,
        "refinement": True,
        "max_iter": 100,
        "roi_circ_threshold": 0,
        "roi_eccentricity_limit": .9,
        "local_max_method": False,
        "area_stop_threshold": .95

    }
    _roi_extraction_params_default_widefield = {
        "elbow_threshold_method": True,
        "elbow_threshold_value": .95,
        "eigen_threshold_method": True,
        "eigen_threshold_value": .9,
        "num_eigen_vector_select": 1,
        "merge_temporal_coef": .99,
        "roi_size_min": 30,
        "roi_size_max": 2000,
        "merge": True,
        "num_rois": 60,
        "fill_holes": True,
        "refinement": True,
        "max_iter": 100,
        "roi_circ_threshold": 0,
        "roi_eccentricity_limit": .9,
        "local_max_method": False,
        "area_stop_threshold": .95

    }
    _time_trace_params_default = {
        "min_neuropil_pixels": 25
    }

    def __init__(self, data_path, save_dir_path, save_dir_already_created, trials=[],
                 parameter_file=False, load_into_mem=True, auto_crop=False,
                 widefield=False, mask_path=None):
        """
        Initializes the object
        Parameters
        ----------
        data_path : str
            This is the path towards the folder containing the data files/folder null
            if save dir is already created
        save_dir_path : str
            save folder location
        save_dir_already_created : bool
            whether to load all the info from the save dir, if this is true data_path
            and trials don't matter
        trials : List[str]
            The list of either names of folders that contain many tiff files or a
            list of tiff files
        """
        self.parameter_file_name = "parameters.json"
        # TODO add loaded trials and all trials parameter here
        # TODO make sure if trial list includes files that aren't valid it works
        self.color_list = DataHandler._color_list

        self.save_dir_path = save_dir_path
        self.time_trace_possibilities_functions = DataHandler.time_trace_possibilities_functions
        if widefield:
            self.time_trace_possibilities_functions = {
                key: DataHandler.time_trace_possibilities_functions[key] for key in
                self.time_trace_possibilities_functions.keys() if "Neuropil" not in key}
        self.rois_loaded = False  # whether roi variables have been created
        self.rois_update_needed = True
        self.widefield = widefield
        self.image_data_mask_flat = None
        if parameter_file:
            self.parameter_file_name = os.path.basename(parameter_file)
            self.save_dir_path = os.path.dirname(parameter_file)
            self.classes = DataHandler._base_class_construct
            self.create_new_save_dir()
            valid = self.load_param_json()
            self.global_params = DataHandler._global_params_default.copy()
            self.global_params["load_into_mem"] = load_into_mem
            if self.dataset_params["single_file_mode"] and not load_into_mem:
                if not os.path.isdir(
                        os.path.join(self.save_dir_path, "temp_files/dataset.zarr")):
                    self.transform_data_to_zarr()
            # these indices are used for which trials to use for roi extract
            self._trials_loaded_indices = [num for num, x in enumerate(self.trials_all)
                                           if x in self.trials_loaded]
            # these are used to determine which trials to calculate time_traces for
            self.trials_loaded_time_trace_indices = [num for num, x in
                                                     enumerate(self.trials_all)
                                                     if x in self.trials_loaded]

            if self.widefield:
                self.load_mask(self.mask_path)
            self.load_dataset(
                [os.path.join(self.dataset_params["dataset_folder_path"], x) for
                 x
                 in self.trials_loaded])
            self.reset_data()
            dask.compute(*self.dataset_trials_filtered_loaded)
            self.global_params["need_recalc_filter_params"] = False
            self.global_params[
                "need_recalc_dataset_params"] = False




        elif save_dir_already_created:  # this loads everything from the save dir
            valid = self.load_param_json()

            if data_path!= "":
                self.dataset_params["dataset_folder_path"] = data_path
            if trials is not None:
                self.dataset_params["trials_loaded"] = trials
                self.dataset_params["trials_all"] = trials
            self.classes = DataHandler._base_class_construct
            self.time_trace_params = DataHandler._time_trace_params_default.copy()
            if self.dataset_params["single_file_mode"] and not self.load_into_mem:
                if not os.path.isdir(
                        os.path.join(self.save_dir_path, "temp_files/dataset.zarr")):
                    self.transform_data_to_zarr()
            # time trace params are not currently saved

            # these indices are used for which trials to use for roi extract
            self._trials_loaded_indices = [num for num, x in enumerate(self.trials_all)
                                           if x in self.trials_loaded]
            # these are used to determine which trials to calculate time_traces for
            self.trials_loaded_time_trace_indices = [num for num, x in
                                                     enumerate(self.trials_all)
                                                     if x in self.trials_loaded]
            if self.widefield:
                self.load_mask(self.mask_path)
            # this loads the dataset and calculate the specified filters
            if not self.load_into_mem:
                self.load_dataset(
                    [os.path.join(self.dataset_params["dataset_folder_path"], x) for
                     x
                     in self.trials_loaded])
                self.reset_data()

                self.dataset_trials_filtered[
                    self._trials_loaded_indices[0]].compute()

                print("Loaded Previous Session")

            else:
                temp = self.global_params["need_recalc_box_params"]

                self.load_dataset(
                    [os.path.join(self.dataset_params["dataset_folder_path"], x) for x
                     in self.trials_loaded])
                self.reset_data()

                self.dataset_trials_filtered[
                    self._trials_loaded_indices[0]].compute()

                self.global_params["need_recalc_box_params"] = temp
            # if there are ROIs saved in the save dir load them and calculate time
            # traces


            if self.rois_exist:
                try:
                    self.load_rois()

                    self.rois_loaded = True
                except:
                    print("ROI loading Failed")
                self.calculate_time_traces()
            if not valid:
                raise FileNotFoundError("Save directory not valid")
        else:
            # start with the default values specified in this class

            self.global_params = DataHandler._global_params_default.copy()
            self.global_params["load_into_mem"] = load_into_mem
            self.dataset_params = DataHandler._dataset_params_default.copy()
            self.filter_params = DataHandler._filter_params_default.copy()
            self.box_params = DataHandler._box_params_default.copy()
            self.box_params_processed = DataHandler._box_params_default.copy()
            self.roi_extraction_params = DataHandler._roi_extraction_params_default.copy()
            if widefield:
                self.roi_extraction_params = DataHandler._roi_extraction_params_default_widefield.copy()

            self.time_trace_params = DataHandler._time_trace_params_default.copy()
            self.classes = DataHandler._base_class_construct
            self.dataset_params["dataset_folder_path"] = data_path
            self.dataset_params["trials_loaded"] = trials
            self.trials_loaded = trials
            self.dataset_params["mask_path"] = mask_path
            if not os.path.isdir(
                    os.path.join(data_path, trials[0])) and len(
                self.trials_loaded) == 1:
                self.dataset_params[
                    "single_file_mode"] = True  # this is different from single dataset mode which is if the len(self.data_list)==1
            if len(self.trials_loaded) > 1:
                self.dataset_params["original_folder_trial_split"] = trials

            if len(self.trials_loaded) == 1:
                self.dataset_params["trial_split"] = True
                self.dataset_params["original_folder_trial_split"] = [trials[0]]
            # these are all files in the data_path directory if the user wants to
            # calculate time traces than for more trials than they originally loaded
            self.trials_all = sorted(os.listdir(data_path))
            if not os.path.isdir(os.path.join(data_path, trials[0])):
                self.trials_all = [x for x in self.trials_all if ".tif" in x]
            self.dataset_params["trials_all"] = self.trials_all

            self.box_params["total_num_time_steps"] = len(trials)
            self.eigen_params = DataHandler._eigen_params_default.copy()

            valid = self.create_new_save_dir()
            if not valid:
                raise FileNotFoundError("Please chose an empty directory for your " +
                                        "save directory")
            if self.dataset_params["single_file_mode"] and not load_into_mem:
                self.transform_data_to_zarr()
            if self.widefield:
                self.load_mask(self.mask_path)

            self.load_dataset(
                [os.path.join(self.dataset_params["dataset_folder_path"], x) for x in
                 self.trials_loaded])

            self._trials_loaded_indices = [num for num, x in enumerate(self.trials_all)
                                           if x in self.trials_loaded]
            self.trials_loaded_time_trace_indices = [num for num, x in
                                                     enumerate(self.trials_all)
                                                     if x in self.trials_loaded]

            if auto_crop:
                try:
                    self.dataset_params["auto_crop"] = False
                    self.dataset_params["crop_x"], self.dataset_params[
                        "crop_y"] = auto_crop_func(self.dataset_list)
                    self.dataset_params["crop_stack"] = True
                    self.reset_data()

                    self.dataset_trials_filtered[self._trials_loaded_indices[0]].compute()
                except IndexError:
                    self.dataset_params["crop_stack"] = False
                    self.reset_data()

                self.dataset_trials_filtered[self._trials_loaded_indices[0]].compute()
            else:
                self.reset_data()

                self.dataset_trials_filtered[self._trials_loaded_indices[0]].compute()
                self.dataset_params["crop_x"] = [0, self.shape[0]]
                self.dataset_params["crop_y"] = [0, self.shape[1]]

            self.save_new_param_json()

    def __del__(self):
        try:
            for x in list(self.__dict__.items()):
                self.__dict__[x[0]] = None
        except TypeError:
            pass

    @property
    def mask_path(self):
        return self.dataset_params["mask_path"]

    @property
    def dataset_trials_filtered_loaded(self):
        """
        Returns the filtered data that is currently loaded
        """
        return [self.dataset_trials_filtered[x] for x in self._trials_loaded_indices]

    @property
    def dataset_trials_loaded(self):
        """
        Returns the unfiltered data that is currently loaded
        """
        return [self.dataset_trials[x] for x in self._trials_loaded_indices]

    @property
    def param_path(self):
        """
        Returns the path to the parameter file
        """
        return os.path.join(self.save_dir_path, self.parameter_file_name)

    @property
    def eigen_vectors_exist(self):
        """
        Returns if the eigen vectors are saved for the current settings
        """

        eigen_dir = os.path.join(self.save_dir_path, "eigen_vectors")

        file_names = [
            "eigen_vectors_box_{}_{}.pickle".format(spatial_box_num, time_box_num)
            for spatial_box_num in range(self.box_params["total_num_spatial_boxes"])
            for time_box_num in range(self.box_params["total_num_time_steps"])]
        return all(pickle_exist(x, output_directory=eigen_dir) for x in file_names)

    @property
    def auto_crop(self):
        return self.dataset_params["auto_crop"]

    @property
    def load_into_mem(self):
        return self.global_params["load_into_mem"]
    @property
    def rois_exist(self):
        """
        Return if the roi save file exists
        """
        return not (self.global_params["need_recalc_eigen_params"] or
                    self.global_params[
                        "need_recalc_roi_extraction_params"] or self.global_params[
                        "need_recalc_box_params"]) and pickle_exist("rois",
                                                                    output_directory=self.save_dir_path)

    @property
    def load_into_mem(self):
        return self.global_params["load_into_mem"]
    @property
    def single_dataset_mode(self):
        # basically whether there are original trials
        return len(self.dataset_params[
                       "original_folder_trial_split"]) == 1
    def load_rois(self):
        """
        Loads the ROIs if they exist and then generates the other variables associated
        with them
        """
        if pickle_exist("rois", output_directory=self.save_dir_path):
            rois = pickle_load("rois", output_directory=self.save_dir_path)
            if type(rois) == dict:
                self.rois_dict = rois
            else:
                self.rois_dict = {(num + 1): {"pixels": roi, "index": num} for num, roi
                                  in enumerate(rois)}
            try:
                self.classes = pickle_load("classes", output_directory=self.save_dir_path)
            except:
                self.classes = {"Unassigned": {"color": (150, 150, 150), "rois": [],
                                               "name": "Unassigned", "editable": False},
                                "Base": {"color": self.color_list[0], "rois": [],
                                         "name": "Base", "editable": True},
                                "Important": {"color": self.color_list[1], "rois": [],
                                              "name": "important", "editable": True}}

            self.gen_roi_display_variables()

    def save_rois(self, rois):
        """
        Saves the rois to files
        Parameters
        ----------
        rois : List[np.ndarray]
            A list pixels of each roi, each pixel is its 1d cordinate

        """
        if os.path.isdir(self.save_dir_path):
            pickle_save(self.rois_dict, "rois", output_directory=self.save_dir_path)
            pickle_save(self.classes, "classes", output_directory=self.save_dir_path)
    def load_param_json(self):
        """
        Loads the parameter json file and saves it to all the parameter values
        """
        return load_param_json(self)
    def save_new_param_json(self):
        """
        Saves the parameters to the parameter file
        """
        return save_new_param_json(self)

    def create_new_save_dir(self):
        """
        Creates a new save directory
        """
        try:
            if not os.path.isdir(self.save_dir_path):
                os.mkdir(self.save_dir_path)
            eigen_vectors_folder_path = os.path.join(self.save_dir_path,
                                                     "eigen_vectors/")

            if not os.path.isdir(eigen_vectors_folder_path):
                os.mkdir(eigen_vectors_folder_path)
            embedding_images_path = os.path.join(self.save_dir_path,
                                                 "embedding_norm_images/")
            if not os.path.isdir(embedding_images_path):
                os.mkdir(embedding_images_path)
            eigen_vectors_folder_path = os.path.join(self.save_dir_path,
                                                     "temp_files/")

            if not os.path.isdir(eigen_vectors_folder_path):
                os.mkdir(eigen_vectors_folder_path)
            return True
        except:
            raise FileNotFoundError("Couldn't create folder please try again")

    def change_global_param(self, param_name, new_value):
        return change_global_param(self, param_name, new_value)

    def change_dataset_param(self, param_name, new_value):
        return change_dataset_param(self, param_name, new_value)

    def change_filter_param(self, param_name, new_value):
        return change_filter_param(self, param_name, new_value)

    def change_box_param(self, param_name, new_value):
        return change_box_param(self, param_name, new_value)

    def change_eigen_param(self, param_name, new_value):
        return change_eigen_param(self, param_name, new_value)

    def change_roi_extraction_param(self, param_name, new_value):
        return change_roi_extraction_param(self, param_name, new_value)

    def transform_data_to_zarr(self):
        return transform_data_to_zarr(self)

    def update_trial_list(self):
        if not self.dataset_params["trial_split"] and self.single_dataset_mode:
            self.trials_loaded = ["0"]
            self.trials_all = self.trials_loaded.copy()
            self.box_params["total_num_time_steps"] = len(self.trials_loaded)
        elif self.single_dataset_mode:
            self.trials_loaded = [str(x) for x in
                                  range(ceil(self.dataset_list[0].shape[0] /
                                             self.dataset_params[
                                                 "trial_length"]))]
            self.trials_all = self.trials_loaded.copy()
            self.box_params["total_num_time_steps"] = len(self.trials_loaded)
        elif not self.single_dataset_mode:
            self.trials_loaded = self.dataset_params["original_folder_trial_split"]
            self.trials_all = self.dataset_params["original_folder_trial_split"]
            self.box_params["total_num_time_steps"] = len(self.trials_loaded)
        if self.single_dataset_mode:
            self._trials_loaded_indices = [num for num, x in
                                           enumerate(self.trials_all)
                                           if x in self.trials_loaded]
            self.trials_loaded_time_trace_indices = [num for num, x in
                                                     enumerate(self.trials_all)
                                                     if x in self.trials_loaded]
    def calculate_dataset(self, ) -> np.ndarray:
        """
        Loads each trial, applying crop and slicing, sets them to self.dataset_trials

        """
        return calculate_dataset(self)
    def compute_trial_filter_step(self, trial_num,loaded_num,dataset=False,save_data=True ):
        """
            Delayed function step that applies filter to a single trial
            Parameters
            ----------
            trial_num : int
                trial num to load in trials_all
            dataset : ndarray
                Dataset to process if false load dataset
            loaded_num :
                trial num in array

            Returns
            -------
            Filtered trial as a np.ndarray
            """
        return compute_trial_filter_step(self, trial_num, loaded_num, dataset,
                                         save_data=save_data)

    # def load_trial_dataset_step(self, trial_num):
    #     """
    #     Delayed function step that loads a single trial
    #     Parameters
    #     ----------
    #     trial_num : int
    #         trial num to load in trials_all
    #
    #     Returns
    #     -------
    #     Trial as a np.ndarray
    #     """
    #     return load_trial_dataset_step(self, trial_num)
    #
    # def load_trial_filter_step(self, trial_num, dataset=False, loaded_num=False):
    #     """
    #     Delayed function step that applies filter to a single trial
    #     Parameters
    #     ----------
    #     trial_num : int
    #         trial num to load in trials_all
    #     dataset : ndarray
    #         Dataset to process if false load dataset
    #
    #     Returns
    #     -------
    #     Filtered trial as a np.ndarray
    #     """
    #     return load_trial_filter_step(self, trial_num, dataset, loaded_num)
    #
    # def calculate_filters(self, trial=0,progress_signal=None, auto_crop=False):
    #     """
    #     Applies filter to each trial, sets them to self.dataset_trials_filtered
    #
    #     Returns
    #     -------
    #     A list of filtered trials
    #     """
    #     return calculate_filters(self, trial,progress_signal, auto_crop)
    def reset_data(self):
        # resets all the data attributes but doesn't reload data itself
        return reset_data(self)

    def load_dataset(self, path):
        # loads a dataset or loads a zarr in
        return load_dataset(self, path)

    def load_mask(self, path):
        return load_mask(self, path)

    @property
    def real_trials(self):
        return not (self.dataset_params["single_file_mode"] or self.dataset_params[
            "trial_split"] or len(self.trials_loaded) == 1)

    def calculate_roi_extraction(self, progress_signal=None):
        """
        Extracts Rois and sets them to self.rois
        """
        return calculate_roi_extraction(self, progress_signal)

    def gen_roi_display_variables(self):
        """
        Generates the other variables associated with displaying ROIs including class ones
        """
        return gen_roi_display_variables(self)

    def gen_class_display_variables(self):
        """
        Generates the class variables associated with displaying ROIs
        """
        return gen_class_display_variables(self)

    def calculate_time_traces(self, report_progress=None):
        """
        Calculates the time traces for every roi in self.rois
        """
        return calculate_time_traces(self, report_progress)

    @property
    def rois(self):

        if self.rois_update_needed:
            keys = list(self.rois_dict.keys())
            for num, x in enumerate(keys):
                self.rois_dict[x]["index"] = num
            self._rois = [self.rois_dict[x]["pixels"] for x in keys]
            self.roi_index_backward = {num: x for num, x in enumerate(keys)}
            self.rois_update_needed = False
        return self._rois

    def assign_roi_class(self, key, new_class, input_key=True):
        if not input_key:
            key = self.roi_index_backward[key]
        self.remove_roi_from_all_classes(key, input_key=True)
        self.classes[new_class]["rois"].append(key)
        self.gen_class_display_variables()

    def remove_roi_from_all_classes(self, key, input_key=True):
        if not input_key:
            key = self.roi_index_backward[key]
        for class_name in self.classes:
            if key in self.classes[class_name]["rois"]:
                self.classes[class_name]["rois"].remove(key)

    def delete_roi(self, key, input_key=True, update_display=True):
        if not input_key:
            key = self.roi_index_backward[key]

        for x in self.time_traces.keys():
            try:
                self.time_traces[x].pop(self.rois_dict[key]["index"])
            except IndexError:
                pass
        try:
            self.roi_time_trace_need_update.pop(self.rois_dict[key]["index"])
        except IndexError:
            pass
        for x in self.classes.keys():
            if key in self.classes[x]["rois"]:
                self.classes[x]["rois"].remove(key)
        del self.rois_dict[key]

        self.rois_update_needed = True
        if update_display:
            self.gen_roi_display_variables()

    def add_new_roi(self, roi_pixels, update=True):
        key = max(list(self.rois_dict.keys())) + 1
        self.rois_dict[key] = {"pixels": roi_pixels}
        self.rois_update_needed = True
        if update:
            self.gen_roi_display_variables()
        self.roi_time_trace_need_update.append(True)
        return key

    def update_roi(self, key, new_pixels, input_key=True):
        if not input_key:
            key = self.roi_index_backward[key]
        self.rois_dict[key]["pixels"] = new_pixels
        self.rois_update_needed = True
        self.gen_roi_display_variables()

    def merge_rois(self, keys, input_key=True):
        if not input_key:
            keys = [self.roi_index_backward[key] for key in keys]
        pixels = set(list(reduce(lambda x, y: x + y,
                                 [list(self.rois_dict[x]["pixels"]) for x in keys])))
        for key in keys:
            self.rois_dict.pop(key, "test")
        index = max(list(self.rois_dict.keys())) + 1
        self.rois_dict[index] = {"pixels": np.array(list(pixels))}
        self.rois_update_needed = True
        self.gen_roi_display_variables()
        return index

    def get_time_trace(self, num, trial=None, trace_type="Mean Florescence"):
        """
        Returns the time trace for a certain roi over all currently selected trials
        Parameters
        ----------
        num : int
            ROI num starts at 1 to num rois

        Returns
        -------
        np.ndarray of the time trace
        """

        return get_time_trace(self, num, trial, trace_type)

    def calculate_filters(self, progress_signal=None):
        if self.global_params["need_recalc_filter_params"] or self.global_params[
            "need_recalc_dataset_params"] or \
            not hasattr(self, "dataset_trials_filtered"):
            self.reset_data()
            self.save_new_param_json()
            self.dataset_trials_filtered[self._trials_loaded_indices[0]].compute()
            self.global_params["need_recalc_filter_params"] =False
            self.global_params[
                "need_recalc_dataset_params"] =False
    def update_selected_trials(self, selected_trials):
        """
        Updates the loaded trials so don't have unnecessary loaded trials
        Parameters
        ----------
        selected_trials
            trials to be currently loaded
        """
        new_selected = [x for x, y in enumerate(self.trials_all) if
                        y in selected_trials]
        for trial_num, _ in enumerate(self.trials_all):
            if trial_num in new_selected and trial_num not in self.trials_loaded_time_trace_indices:
                self.dataset_trials_filtered[trial_num] = self.load_trial_filter_step(
                    trial_num).compute()
                self.trials_loaded_time_trace_indices.append(trial_num)
            if trial_num not in new_selected and trial_num in self.trials_loaded_time_trace_indices \
                    and trial_num not in self._trials_loaded_indices:
                self.dataset_trials_filtered[trial_num] = False

            if trial_num not in new_selected and trial_num in self.trials_loaded_time_trace_indices:
                self.trials_loaded_time_trace_indices.remove(trial_num)

    def get_eigen_vector(self, box_num, vector_num, trial_num):
        vector = loadEigenVectors(spatial_box_num=box_num, time_box_num=trial_num,
                                  save_dir=self.save_dir_path).compute()
        spatial_box = SpatialBox(box_num,
                                 total_boxes=self.box_params["total_num_spatial_boxes"],
                                 image_shape=self.shape,
                                 spatial_overlap=self.box_params[
                                                     "spatial_overlap"] // 2)
        return vector[:, vector_num].reshape(spatial_box.shape)

    def genRoiFromPoint(self, point, growth_factor=1.0):
        """
        Generates an roi from an initial point in the image
        Parameters
        ----------
        point : List[int, int]
            2d cordinates of a point in the image
        growth_factor : float
            how much to grow the roi

        Returns
        -------
        the pixels in said roi in there 1d form
        """
        return genRoiFromPoint(self, point, growth_factor)

    def calculate_statistics(self):
        return calculate_statistics(self)

    def export(self, matlab=False, background_images=["max", "mean", "eigen_norm", "blank"],
               color_maps=['gray']):
        return export(self, matlab, background_images, color_maps)

    def save_image(self, image, path):
        return save_image(self, image, path)

    def delete_roi_vars(self):
        # self.rois = []
        self.rois_dict = {}
        self.rois_update_needed = True
        for key in self.classes.keys():
            self.classes[key]["rois"] = []
        self.rois_loaded = False
        self.roi_max_cord_list = None
        self.roi_min_cord_list = None
        self.pixel_with_rois_flat = None
        self.pixel_with_rois_color_flat = None
        self.time_traces = []
        self.edge_roi_image_flat = None
        self.pixel_with_rois_color = None
        self.eigen_norm_image = None

    def export_overlapping_rois(self, current_roi_num_1, current_roi_num_2):
        """THis is only used in dev version it allows the user to export time trace of all pixels in this roi and in overlapping rois"""
        export_overlapping_rois(self, current_roi_num_1, current_roi_num_2)



import numpy as np
from scipy.ndimage import binary_dilation


def calculate_neuropil(image_shape, roi_list, roi_mask_flat, min_pixels=25,
                       neuropil_boundary=2):
    neuropil = []
    for roi in roi_list:
        original_size = len(roi)
        roi_mask = np.reshape(roi_mask_flat.copy(), image_shape)
        roi_mask[roi_mask > 0] = 1
        roi_opposite_mask = (roi_mask == 0)
        roi_image = np.zeros(roi_mask_flat.shape)
        roi_image[roi] = 1

        roi_image = np.reshape(roi_image, image_shape)
        roi_image_original = roi_image.copy()
        for _ in range(int(neuropil_boundary)):
            roi_image_original = binary_dilation(input=roi_image_original).copy()
        while np.count_nonzero((
                                       roi_image - roi_mask - roi_image_original) > 0) - original_size < min_pixels:
            roi_image = binary_dilation(input=roi_image)
        neuropil.append(np.nonzero(
            np.reshape((roi_image - roi_mask - roi_image_original) > 0, (-1))))
    return neuropil



import dask
import numpy as np
from skimage import feature
from skimage.transform import rescale, hough_circle, hough_circle_peaks

from cidan.LSSC.functions.data_manipulation import pixel_num_to_2d_cord


def filterRoiList(roi_list, image_shape, threshold=.7):
    """
    Filters a list of ROIS based on looking at surface area vs surface area of a similar sized circle of each roi
    Parameters
    ----------
    roi_list
    image_shape
    threshold

    Returns
    -------
    list of threshold_values
    """
    if len(roi_list)==0:
        return []
    out_list = [filterRoi(x, image_shape=image_shape, threshold=threshold) for x in
                roi_list]
    out = dask.compute(*out_list)
    # return [int(x[0]) for x in out]
    return [int(x) for x in out / max(out) * 100]


# @dask.delayed
def filterRoi(roi, image_shape, threshold):
    roi_2d_cord = pixel_num_to_2d_cord(roi, volume_shape=image_shape)
    roi_max_cord = np.max(roi_2d_cord, axis=1)
    roi_min_cord = np.min(roi_2d_cord, axis=1)
    newShape = roi_max_cord - roi_min_cord + np.array([10, 10])
    image = np.zeros(newShape)
    for cord in zip(
            *list(roi_2d_cord - np.reshape(roi_min_cord - np.array([2, 2]), (2, 1)))):
        image[cord[0], cord[1]] = 1

    image = rescale(image, 2, anti_aliasing=False)
    image_canny = feature.canny(image)
    # radii =range(int(np.min(roi_max_cord - roi_min_cord)*.8),math.ceil(np.min(roi_max_cord - roi_min_cord)*1.2)+1,1)
    radii = [np.min(roi_max_cord - roi_min_cord)]
    result = hough_circle_peaks(hough_circle(image_canny, radii), radii,
                                total_num_peaks=1)
    # result = hough(image_canny, accuracy=20, threshold=0,
    #                        min_size=int(np.min(roi_max_cord - roi_min_cord)/1.7), max_size=int(np.max(roi_max_cord - roi_min_cord)*2.1))
    try:
        return result[0][0]
    except:
        return 0
    result.sort(order='accumulator')
    result_filtered = filter(lambda x: not any([y == 0 for y in x]), reversed(result))
    sqrt_2 = 2 ** .5
    for x in result_filtered:
        if (int(np.max([x[3], x[4]])) <= np.max(roi_max_cord - roi_min_cord) * sqrt_2):
            if (np.min([x[3], x[4]]) >= np.min(roi_max_cord - roi_min_cord)):
                return list(x) + [(roi_max_cord - roi_min_cord)]

    return [0.01]
    surface_pixels = np.count_nonzero(image_canny)
    return ((((np.min(roi_max_cord - roi_min_cord)) * 3.14))) / (surface_pixels / 4)



import shutil
import time

from PySide2.QtWidgets import QApplication

from cidan.GUI.Data_Interaction.loadDataset import load_new_dataset, load_prev_session
from cidan.GUI.MainWindow import MainWindow


def test_main():
    try:
        app = QApplication([])
        app.setApplicationName("cidan")
    except RuntimeError:
        pass
    widget = MainWindow(dev=True, preload=False)

    main_widget = widget.table_widget
    load_new_dataset(main_widget, file_path="test_files/small_dataset1.tif",
                     save_dir_path="test_files/save_dir", load_into_mem=False)
    main_widget.open_dataset_thread.wait()
    main_widget.init_w_data()
    assert main_widget.data_handler.shape == [400, 150]
    assert main_widget.data_handler.rois_loaded == False
    assert main_widget.data_handler.trials_loaded == ["0"]
    assert main_widget.data_handler.trials_all == ["0"]
    assert main_widget.data_handler.trials_loaded_time_trace_indices == [0]
    assert main_widget.data_handler._trials_loaded_indices == [0]
    for tab in main_widget.tabs:
        assert tab.image_view.image_item.image.shape[0] == 400
        assert tab.image_view.image_item.image.shape[1] == 150
    data_handler = main_widget.data_handler
    data_handler.change_dataset_param("crop_x", [0, 400])
    data_handler.change_dataset_param("crop_stack", True)
    data_handler.change_filter_param("hist_eq", True)
    main_widget.thread_list[0].run()
    assert main_widget.data_handler.shape == [400, 150]

    data_handler.change_filter_param("hist_eq", False)
    data_handler.change_filter_param("median_filter", True)
    data_handler.change_roi_extraction_param("roi_circ_threshold", 0)
    main_widget.thread_list[1].run()
    main_widget.exportStuff()
    assert main_widget.data_handler.shape == [200, 150]
    assert len(main_widget.data_handler.rois) != 0
    assert main_widget.data_handler.rois_loaded == True
    roi_image_view = main_widget.tabs[1].image_view
    roi_image_view.zoomRoi(2)
    roi_2_pixels = main_widget.data_handler.rois[1]
    for x in range(14):
        roi_image_view.pixel_paint(x, 2)
    main_widget.tabs[1].roi_list_module.set_current_select(2)
    assert main_widget.tabs[1].modify_roi(2, add_subtract="add", override=True)
    assert list(roi_2_pixels) != list(main_widget.data_handler.rois[1])
    assert not main_widget.tabs[1].modify_roi(2, add_subtract="add", override=True)

    for x in range(14):
        roi_image_view.pixel_paint(x, 2)
    main_widget.tabs[1].roi_list_module.set_current_select(2)
    assert main_widget.tabs[1].modify_roi(2, add_subtract="subtract", override=True)
    assert list(roi_2_pixels) == list(main_widget.data_handler.rois[1])
    num_rois = len(data_handler.rois)
    main_widget.tabs[1].delete_roi(2)
    assert num_rois - 1 == len(data_handler.rois)
    assert list(roi_2_pixels) != list(main_widget.data_handler.rois[1])
    roi_image_view.setBrushSize(5)
    for x in range(14):
        roi_image_view.pixel_paint(x, 2)
    main_widget.tabs[1].add_new_roi()
    assert num_rois == len(data_handler.rois)
    for x in range(14):
        roi_image_view.pixel_paint(x, 2)
    roi_image_view.select_mode = "subtract"
    for x in range(14):
        roi_image_view.pixel_paint(x, 2)
    assert not main_widget.tabs[1].modify_roi(2, add_subtract="add", override=True)
    # assert roi_image_view.magic_wand(50, 120)

    roi_image_view.set_background("", "Mean Image")
    roi_image_view.set_background("", "Blank Image")
    roi_image_view.set_background("", "Eigen Norm Image")
    roi_image_view.set_image("", "Blob")
    main_widget.tabs[2].roi_list_module.set_current_select(2)
    main_widget.tabs[2].roi_list_module.set_current_select(1)
    main_widget.tabs[2].plot_type_input.input_box.setCurrentIndex(1)
    main_widget.tabs[2].plot_by_input.input_box.setCurrentIndex(1)

    main_widget.tabs[2].roi_list_module.set_current_select(1)
    main_widget.tabs[2].plot_type_input.input_box.setCurrentIndex(0)
    main_widget.tabs[2].time_trace_type.input_box.setCurrentIndex(1)
    main_widget.tabs[2]._time_trace_trial_select_list.selectAll()
    main_widget.tabs[2].update_time_traces()
    main_widget.tabs[2].time_trace_type.input_box.setCurrentIndex(0)
    main_widget.tabs[2].roi_list_module.set_current_select(2)
    main_widget.tabs[2].roi_list_module.set_current_select(1)
    main_widget.tabs[2].plot_type_input.input_box.setCurrentIndex(1)
    main_widget.tabs[2].plot_by_input.input_box.setCurrentIndex(1)

    main_widget.tabs[2].roi_list_module.set_current_select(1)
    main_widget.tabs[2].plot_type_input.input_box.setCurrentIndex(0)
    main_widget.tabs[2].time_trace_type.input_box.setCurrentIndex(1)
    data_handler.change_filter_param("pca", True)
    main_widget.thread_list[0].run()
    print("test")
    main_widget.thread_list[1].run()
    assert main_widget.data_handler.shape == [200, 150]
    assert len(main_widget.data_handler.rois) != 0
    assert main_widget.data_handler.rois_loaded == True
    roi_image_view = None
    main_widget = None
    data_handler = None
    widget = None

    widget = MainWindow(dev=False, preload=False)
    main_widget = widget.table_widget
    load_prev_session(main_widget,
                      save_dir_path="test_files/save_dir")
    main_widget.open_dataset_thread.wait()
    assert len(main_widget.data_handler.rois) != 0
    assert main_widget.data_handler.shape == [200, 150]
    assert main_widget.data_handler.rois_loaded == True
    data_handler = main_widget.data_handler
    data_handler.change_dataset_param("crop_x", [0, 200])
    data_handler.change_dataset_param("crop_stack", True)
    data_handler.change_filter_param("hist_eq", True)
    main_widget.thread_list[0].run()
    assert data_handler.rois_loaded == False
    load_new_dataset(main_widget, file_path="test_files/",
                     save_dir_path="test_files/save_dir",
                     trials=["small_dataset1.tif", "small_dataset2.tif"])
    main_widget.open_dataset_thread.wait()
    time.sleep(15)
    assert main_widget.data_handler.shape == [400, 150]
    assert main_widget.data_handler.trials_loaded == ["small_dataset1.tif",
                                                      "small_dataset2.tif"]
    assert main_widget.data_handler.trials_all == ["small_dataset1.tif",
                                                   "small_dataset2.tif"]
    assert main_widget.data_handler.trials_loaded_time_trace_indices == [0, 1]
    assert main_widget.data_handler._trials_loaded_indices == [0, 1]
    try:
        app.quit()
    except:
        pass
    shutil.rmtree("test_files/save_dir")



from itertools import groupby

import numpy as np


def trial_Pnoise(X, **kwargs):
    W = kwargs['W'] if 'W' in kwargs else 25
    step = kwargs['step'] if 'step' in kwargs else 10
    N = X.shape[1]

    # calculate P_Noise
    #     P_Noise = utils.make2dList(X.shape[0], 2)
    #     P_Noise = [None] * X.shape[0]
    P_Noise = np.zeros((X.shape[0], 2))
    for trial in range(X.shape[0]):
        record_std = []
        record_mean = []
        for w in range((N - W) // step + 1):
            window = w * step + np.linspace(0, W, W, endpoint=False, dtype=int)
            #             record.append(np.std(X[trial,window], ddof=1))
            record_std.append(np.std(X[trial, window]))
            record_mean.append(np.mean(X[trial, window]))
        medIdx = record_std.index(
            np.percentile(record_std, 25, interpolation='nearest'))
        P_Noise[trial, [0, 1]] = [record_mean[medIdx], record_std[medIdx]]
    #         P_Noise[trial] = np.median(record)

    return P_Noise


def trial_PSNR(X, **kwargs):
    Peak_Sig = np.amax(X, axis=1, keepdims=True)
    #     P_Sig = np.linalg.norm(X_Str[0], keepdims=True, axis=1)**2 / N
    if 'Pnoise' in kwargs:
        P_Noise = kwargs['Pnoise']
    else:
        W = kwargs['W'] if 'W' in kwargs else 25
        step = kwargs['step'] if 'step' in kwargs else 10
        P_Noise = trial_Pnoise(X, W=W, step=step)[:, 1]

    #     SNR = np.log10(Peak_Sig/P_Noise)
    SNR = Peak_Sig / P_Noise
    return SNR.flatten()


def trial_active(X, **kwargs):
    activeTrial_list = []
    # nSTD - # of std above mean for activity threshold
    # width - # of time frames for activity threshold
    nSTD = kwargs['n'] if 'n' in kwargs else 1
    width = kwargs['w'] if 'w' in kwargs else 4

    noise_stats = trial_Pnoise(X, W=25, step=10)

    for trialId in range(X.shape[0]):
        trial = X[trialId, :]
        meanNoise, stdNoise = noise_stats[trialId, :]
        T = trial > (meanNoise + nSTD * stdNoise)
        for k, g in groupby(T):
            if k:
                length = sum(1 for _ in g)
                if length >= width:
                    activeTrial_list.append(trialId)
                    break
    #     temp = [noise_stats[i] for i in activeTrial_list]
    #     return X[activeTrial_list,:], noise_stats[activeTrial_list,:]
    return noise_stats, activeTrial_list



from cidan.LSSC.functions.pickle_funcs import *
def test_pickle_funcs():
    test_dir = "test_pickle"
    pickle_set_dir(test_dir)
    if not os.path.isdir(test_dir):
        os.mkdir(test_dir)
    pickle_clear(trial_num=0)
    assert not pickle_exist("test", trial_num=0)
    obj = "pickle save"
    pickle_save(obj, "test",trial_num=0)
    assert len([f for f in os.listdir("{0}/0/".format(test_dir))]) == 1
    assert pickle_load("test", trial_num=0)==obj
    assert pickle_exist("test", trial_num=0)
    pickle_clear(trial_num=0)
    assert not pickle_exist("test", trial_num=0)
    assert len([f for f in os.listdir("{0}/0/".format(test_dir))]) == 0
    os.rmdir("{0}/0/".format(test_dir))
    os.rmdir(test_dir)




import logging
import sys

from qtpy import QtWidgets

from cidan.GUI.Data_Interaction.Signals import StrIntSignal
from cidan.GUI.Data_Interaction.Thread import Thread

logger = logging.getLogger("cidan.GUI.Data_Interaction.CalculateSingleTrialThread")


class CalculateSingleTrialThread(Thread):

    def __init__(self, main_widget):
        super().__init__(main_widget)
        self.trial_num = 0
        self.end_function = min
        self.signal.sig.connect(lambda x: self.endThread(x))

    def run(self):
        if self.main_widget.dev:

            self.data_handler.dataset_trials_filtered_loaded[self.trial_num].compute()
            self.signal.sig.emit(True)
        else:
            try:
                self.data_handler.dataset_trials_filtered_loaded[self.trial_num].compute()
            except Exception as e:
                logger.error(e)
                print("Unexpected error:", sys.exc_info()[0])
                self.main_widget.console.updateText("Unexpected error: " +
                                                    sys.exc_info()[0])
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage("Unexpected error: " + str(e))
                self.signal.sig.emit(False)

    def runThread(self, trial_num, function):

        if not any([x.isRunning() for x in self.main_widget.thread_list]):
            self.trial_num = trial_num
            self.end_function = function
            self.start()
        else:
            print(
                "Previous process in process, please wait to start new one till "
                "finished")

    def endThread(self, success):
        self.end_function()




import logging

import dask
import h5py
import numpy as np
import zarr
from dask import delayed
from tifffile import tifffile

from cidan.LSSC.functions.data_manipulation import load_tif_stack, filter_stack, \
    applyPCA, load_tif_stack_trial, slice_crop_stack
from cidan.LSSC.functions.pickle_funcs import *

logger1 = logging.getLogger("cidan.DataHandler")


def reset_data(self):
    self.update_trial_list()
    self.dataset_trials_filtered = [False] * len(self.trials_all)
    for num, trial_num in enumerate(self._trials_loaded_indices):
        self.dataset_trials_filtered[
                            trial_num] = self.compute_trial_filter_step(
                            trial_num,
                            loaded_num=num)
    if self.filter_params["pca"]:
        self.pca_decomp = [False] * len(self.trials_loaded_time_trace_indices)
    # for trial_num in self._trials_loaded_indices:
        # self.dataset_trials_filtered[trial_num] = zarr.open(
        #     os.path.join(self.save_dir_path,
        #                  'temp_files/%s.zarr' % self.trials_all[trial_num]),
        #     mode="r")
        # if self.filter_params["pca"]:
        #     self.pca_decomp[trial_num] = zarr.open(
        #         os.path.join(self.save_dir_path,
        #                      'temp_files/%s_pca.zarr' % self.trials_all[trial_num]),
        #         mode="r")
    # max_image_zarr = zarr.open(
    #     os.path.join(self.save_dir_path,
    #                  'temp_files/max.zarr'),
    #     mode="r")
    self.max_images = [False] * len(self.trials_loaded_time_trace_indices)
    # mean_image_zarr = zarr.open(
    #     os.path.join(self.save_dir_path,
    #                  'temp_files/mean.zarr'),
    #     mode="r")
    self.mean_images = [False] * len(self.trials_loaded_time_trace_indices)
    # for x in range(len(self._trials_loaded_indices)):
    #     self.max_images[x] = max_image_zarr[:, :, x]
    #     self.mean_images[x] = mean_image_zarr[:, :, x]
    # self.shape = [
    #     self.dataset_trials_filtered[self._trials_loaded_indices[0]].shape[1],
    #     self.dataset_trials_filtered[self._trials_loaded_indices[0]].shape[2]]
    # if self.dataset_params["crop_x"][1] == 0:
    #     self.dataset_params["crop_x"][1] = self.shape[0]
    #     self.dataset_params["crop_y"][1] = self.shape[1] # TODO make sure to re-add this in somewhere
    # self.mean_images = [np.mean(x[:], axis=0) for x in
    #                     self.dataset_trials_filtered_loaded]
    #
    # self.max_images = [np.max(x[:], axis=0) for x in
    #                    self.dataset_trials_filtered_loaded]
    self.global_params["need_recalc_filter_params"] = False
    self.global_params["need_recalc_dataset_params"] = False


def transform_data_to_zarr(self):
    # used in large data mode to turn a single tiff file into a dataset where it is easy to extract only the time frams that are necessary
    image = tifffile.imread(os.path.join(self.dataset_params["dataset_folder_path"],
                                         self.dataset_params[
                                             "original_folder_trial_split"]))
    z1 = zarr.open(os.path.join(self.save_dir_path,
                                'temp_files/dataset.zarr'), mode='w',
                   shape=image.shape,
                   chunks=(4, image.shape[1],image.shape[2]), dtype=image.dtype)
    z1[:] = image


def load_dataset(self, path_list):
    # TODO add in zarr load support to here
    if self.load_into_mem:
        if self.single_dataset_mode:

            self.dataset_list = [load_tif_stack(path_list[0],
                                                mask_flat=self.image_data_mask_flat if self.widefield else None,
                                                convert_to_32=False)]
        else:
            self.dataset_list = dask.compute(
                *[delayed(load_tif_stack)(x,
                                          mask_flat=self.image_data_mask_flat if self.widefield else None)
                  for x in path_list])
        self.total_size = [self.dataset_list[0].shape[1], self.dataset_list[0].shape[2]]
    else:
        if self.single_dataset_mode:
            self.dataset_list = [load_tif_stack(path_list[0],
                                                mask_flat=self.image_data_mask_flat if self.widefield else None,
                                                convert_to_32=False)]
        else:
            self.dataset_list = dask.compute(
                *[delayed(load_tif_stack)(x,
                                          mask_flat=self.image_data_mask_flat if self.widefield else None)
                  for x in path_list])
        self.total_size = [self.dataset_list[0].shape[1],self.dataset_list[0].shape[2]]


def calculate_dataset(self) -> np.ndarray:
    """
    Loads each trial, applying crop and slicing, sets them to self.dataset_trials

    """
    # TODO make it so it does't load the dataset every time

    dataset_trials = [False] * len(self.trials_all)
    for trial_num in self._trials_loaded_indices:
        dataset_trials[trial_num] = self.load_trial_dataset_step(trial_num)
    dataset_trials = dask.compute(*dataset_trials)
    self.global_params["need_recalc_dataset_params"] = False
    self.shape = [dataset_trials[self._trials_loaded_indices[0]].shape[1],
                  dataset_trials[self._trials_loaded_indices[0]].shape[2]]
    print(dataset_trials[0])
    if self.dataset_params["crop_x"][1] == 0:
        self.dataset_params["crop_x"][1] = self.shape[0]
        self.dataset_params["crop_y"][1] = self.shape[1]
    print("Finished Calculating Dataset")
    return dataset_trials

@delayed
def load_dataset_trial(self, trial_num):
    """
    Delayed function step that loads a single trial

    should only be used in large data mode
    Parameters
    ----------
    trial_num : int
        trial num to load in trials_all

    Returns
    -------
    Trial as a np.ndarray
    """
    # TODO make these params correct
    total_size, stack = load_tif_stack_trial(
        path=os.path.join(self.dataset_params["dataset_folder_path"],
                          self.trials_all[trial_num] if not self.dataset_params[
                              "trial_split"] else self.dataset_params[
                              "original_folder_trial_split"]),
        # filter=False,
        # median_filter=False,
        # median_filter_size=(1, 3, 3),
        # z_score=False,
        # slice_stack=self.dataset_params[
        #     "slice_stack"],
        # slice_start=self.dataset_params[
        #     "slice_start"],
        #
        # slice_every=self.dataset_params[
        #     "slice_every"],
        # crop_stack=self.dataset_params[
        #     "crop_stack"],
        # crop_x=self.dataset_params[
        #     "crop_x"],
        # crop_y=self.dataset_params[
        #     "crop_y"], load_into_mem=self.load_into_mem,
        trial_split=self.dataset_params["trial_split"],
        trial_split_length=self.dataset_params["trial_length"], trial_num=trial_num,
        # zarr_path=False if not self.dataset_params["single_file_mode"] else
        # os.path.join(self.save_dir_path, "temp_files/dataset.zarr")
    )
    self.total_size = total_size
    return stack

@delayed
def compute_trial_filter_step(self, trial_num, loaded_num,dataset=False, save_data=True ):
    """
    Delayed function step that applies filter to a single trial
    Parameters
    ----------
    trial_num : int
        trial num to load in trials_all
    dataset : ndarray
        Dataset to process if false load dataset
    loaded_num :
        trial num in array

    Returns
    -------
    Filtered trial as a np.ndarray
    """
    # NEED to add the data spliting code to here

    if self.dataset_params[
        "trial_split"]:
        cur_stack = self.dataset_list[0]
        cur_stack = cur_stack[trial_num * self.dataset_params[
        "trial_length"]:(loaded_num + 1) * self.dataset_params[
        "trial_length"],
        :, :]
    else:
        cur_stack = self.dataset_list[trial_num][:]
    cur_stack =cur_stack.astype(np.float32)

    cur_stack = slice_crop_stack(cur_stack,slice_stack=self.dataset_params[
            "slice_stack"],
        slice_start=self.dataset_params[
            "slice_start"],

        slice_every=self.dataset_params[
            "slice_every"],
        crop_stack=self.dataset_params[
            "crop_stack"],
        crop_x=self.dataset_params[
            "crop_x"],
        crop_y=self.dataset_params[
            "crop_y"])

    cur_stack = filter_stack(

        stack=cur_stack,
        median_filter_size=(self.filter_params[
                                "median_filter_size"],
                            self.filter_params[
                                "median_filter_size"],
                            self.filter_params[
                                "median_filter_size"]),
        median_filter=self.filter_params[
            "median_filter"],
        z_score=self.filter_params["z_score"],
        hist_eq=self.filter_params["hist_eq"],
        localSpatialDenoising=self.filter_params["localSpatialDenoising"])
    self.shape = [cur_stack.shape[1], cur_stack.shape[2]]

    if not self.load_into_mem:

        z1 = zarr.open(os.path.join(self.save_dir_path,
                                    'temp_files/%s.zarr' % self.trials_all[
                                        trial_num]), mode='w',
                       shape=cur_stack.shape,
                       chunks=(32, 128, 128), dtype=np.float32)
        z1[:] = cur_stack

            # self.temporal_correlation_images[
            #     loaded_num] = calculate_temporal_correlation(cur_stack).compute()




    self.mean_images[loaded_num] = np.mean(cur_stack, axis=0)
    self.max_images[loaded_num] = np.max(cur_stack, axis=0)
            # self.temporal_correlation_images[
            #     loaded_num] = calculate_temporal_correlation(
            #     cur_stack).compute()
    if self.filter_params["pca"]:
        pca = applyPCA(cur_stack, self.filter_params["pca_threshold"],
                       mask_flat=self.image_data_mask_flat)
        if self.load_into_mem:
            self.pca_decomp[loaded_num] = pca
        else:
            z2 = zarr.open(os.path.join(self.save_dir_path,
                                        'temp_files/%s_pca.zarr' % self.trials_all[
                                            trial_num]), mode='w',
                           shape=pca.shape,
                           chunks=(3, 64, 64))
            z2[:] = pca
            self.pca_decomp[loaded_num] = z2

    # with open(os.path.join(self.save_dir_path,
    #                        'temp_files/filter/%s' % self.trials_all[
    #                            trial_num]), "w") as f:
    #     f.write("done")
    # printProgressBarFilter(
    #     total_num_spatial_boxes=self.box_params["total_num_spatial_boxes"],
    #     total_num_time_steps=self.box_params["total_num_time_steps"],
    #     save_dir=self.save_dir_path, progress_signal=self.progress_signal)

    if self.load_into_mem:
        if save_data:
            self.dataset_trials_filtered[trial_num] = cur_stack
        return cur_stack
    else:
        if save_data:
            self.dataset_trials_filtered[trial_num] = z1
        return z1


def load_mask(self, path):
    if ".mat" in path:
        try:
            with h5py.File(path, 'r') as f:
                keys = list(f.keys())
                if "mask" in keys:
                    key = "mask"
                else:
                    key = keys[0]
                mask = np.array(f[key]).astype(bool)
                self.image_data_mask_flat = mask.reshape((-1, 1))
                return True
        except Exception as a:
            print(a)
            print(
                "Loading of mat file failed please make sure the mask is in variable called mask and that it is the right size")
    elif ".tif" in path:
        image = tifffile.imread(path)
        mask = image.astype(bool).reshape((-1, 1))
        self.image_data_mask_flat = mask
    else:
        print("Please input a mat file for the mask")
        # raise warnings.warn_explicit("Please enter a valid mat file for the mask")
        return False

# def calculate_filters(self, progress_signal=None, auto_crop=False):
#     """
#     Applies filter to each trial, sets them to self.dataset_trials_filtered
#
#     Returns
#     -------
#     A list of filtered trials
#     """
#     if not self.auto_crop:
#         self.dataset_params["auto_crop"] = auto_crop
#     if self.global_params["need_recalc_filter_params"] or self.global_params[
#         "need_recalc_dataset_params"] or \
#             not hasattr(self, "dataset_trials_filtered"):
#         self.progress_signal = progress_signal
#         print("Started Calculating Filters")
#         reset_data(self)
#
#         save_dir = self.save_dir_path
#         if not os.path.isdir(os.path.join(save_dir, "temp_files/filter")):
#             os.mkdir(os.path.join(save_dir, "temp_files/filter"))
#         filelist = [f for f in
#                     os.listdir(os.path.join(save_dir, "temp_files/filter"))]
#         for f in filelist:
#             os.remove(
#                 os.path.join(os.path.join(save_dir, "temp_files/filter"), f))
#
#         printProgressBarFilter(
#             total_num_spatial_boxes=self.box_params["total_num_spatial_boxes"],
#             total_num_time_steps=self.box_params["total_num_time_steps"],
#             save_dir=self.save_dir_path, progress_signal=self.progress_signal)
#         if self.auto_crop:
#             self.suggested_crops = [[[0, 0], [0, 0]] for _ in self.trials_all]
#
#
#         self.temporal_correlation_images = [False] * len(
#             self._trials_loaded_indices)
#         if not self.load_into_mem:
#             for num, trial_num in enumerate(self._trials_loaded_indices):
#                 self.dataset_trials_filtered[
#                     trial_num] = self.load_trial_filter_step(
#                     trial_num, self.load_trial_dataset_step(trial_num),
#                     loaded_num=num)
#                 if num % 3 == 0:
#                     self.dataset_trials_filtered = list(
#                         dask.compute(*self.dataset_trials_filtered))
#
#             self.dataset_trials_filtered = list(
#                 dask.compute(*self.dataset_trials_filtered))
#         else:
#             for num, trial_num in enumerate(self._trials_loaded_indices):
#                 self.dataset_trials_filtered[
#                     trial_num] = self.load_trial_filter_step(
#                     trial_num, self.load_trial_dataset_step(trial_num),
#                     loaded_num=num)
#             if num % 7 == 0:
#                 self.dataset_trials_filtered = list(
#                     dask.compute(*self.dataset_trials_filtered))
#             self.dataset_trials_filtered = list(
#                 dask.compute(*self.dataset_trials_filtered))
#         self.shape = [
#             self.dataset_trials_filtered[self._trials_loaded_indices[0]].shape[1],
#             self.dataset_trials_filtered[self._trials_loaded_indices[0]].shape[2]]
#         if self.dataset_params["crop_x"][1] == 0:
#             self.dataset_params["crop_x"][1] = self.shape[0]
#             self.dataset_params["crop_y"][1] = self.shape[1]
#         if self.auto_crop:
#             crop = [[max([x[0][0] for x in self.suggested_crops]),
#                      -max([x[0][1] for x in self.suggested_crops])],
#                     [max([x[1][0] for x in self.suggested_crops]),
#                      -max([x[1][1] for x in self.suggested_crops])]]
#
#             self.dataset_params["crop_x"][0] = 0 + \
#                                                crop[0][0]
#             self.dataset_params["crop_x"][1] = self.total_size[0] + \
#                                                crop[0][1]
#             self.dataset_params["crop_y"][0] = 0 + \
#                                                crop[1][0]
#             self.dataset_params["crop_y"][1] = self.total_size[1] + \
#                                                crop[1][1]
#             if crop[0][1] == 0:
#                 crop[0][1] = self.total_size[0]
#             if crop[1][1] == 0:
#                 crop[1][1] = self.total_size[1]
#             for trial_num in self._trials_loaded_indices:
#                 self.dataset_trials_filtered[trial_num] = \
#                     self.dataset_trials_filtered[trial_num][
#                     :, crop[0][0]:crop[0][1], crop[1][0]:crop[1][1]]
#                 self.mean_images[trial_num] = self.mean_images[trial_num][
#                                               crop[0][0]:crop[0][1],
#                                               crop[1][0]:crop[1][1]]
#                 self.max_images[trial_num] = self.max_images[trial_num][
#                                              crop[0][0]:crop[0][1],
#                                              crop[1][0]:crop[1][1]]
#                 # self.temporal_correlation_images[trial_num] = \
#                 #     self.temporal_correlation_images[trial_num][
#                 #     crop[0][0]:crop[0][1], crop[1][0]:crop[1][1]]
#                 if self.filter_params["pca"]:
#                     self.pca_decomp[trial_num] = self.pca_decomp[trial_num][
#                                                  :, crop[0][0]:crop[0][1],
#                                                  crop[1][0]:crop[1][1]]
#             self.dataset_params["crop_stack"] = True
#             self.dataset_params["auto_crop"] = False
#             self.shape = [
#                 self.dataset_trials_filtered[self._trials_loaded_indices[0]].shape[
#                     1],
#                 self.dataset_trials_filtered[self._trials_loaded_indices[0]].shape[
#                     2]]
#
#         mean_image_stack = np.dstack(self.mean_images)
#         with open(os.path.join(self.save_dir_path,
#                                'temp_files/filter/mean'), "w") as f:
#             f.write("done")
#         printProgressBarFilter(
#             total_num_spatial_boxes=self.box_params["total_num_spatial_boxes"],
#             total_num_time_steps=self.box_params["total_num_time_steps"],
#             save_dir=self.save_dir_path, progress_signal=self.progress_signal)
#
#         max_image_stack = np.dstack(self.max_images)
#         with open(os.path.join(self.save_dir_path,
#                                'temp_files/filter/max'), "w") as f:
#             f.write("done")
#         mean_image_zarr = zarr.open(os.path.join(self.save_dir_path,
#                                                  'temp_files/mean.zarr'), mode='w',
#                                     shape=mean_image_stack.shape,
#                                     chunks=(40, 40, 1))
#         mean_image_zarr[:] = mean_image_stack
#
#         max_image_zarr = zarr.open(os.path.join(self.save_dir_path,
#                                                 'temp_files/max.zarr'), mode='w',
#                                    shape=mean_image_stack.shape,
#                                    chunks=(40, 40, 1))
#         max_image_zarr[:] = max_image_stack
#         printProgressBarFilter(
#             total_num_spatial_boxes=self.box_params["total_num_spatial_boxes"],
#             total_num_time_steps=self.box_params["total_num_time_steps"],
#             save_dir=self.save_dir_path, progress_signal=self.progress_signal)
#
#         # self.mean_images = [np.mean(x[:], axis=0) for x in
#         #                     self.dataset_trials_filtered_loaded]
#         #
#         # self.max_images = [np.max(x[:], axis=0) for x in
#         #                    self.dataset_trials_filtered_loaded]
#
#         # self.temporal_correlation_image = calculate_temporal_correlation(self.dataset_filtered)
#         self.global_params["need_recalc_filter_params"] = False
#         self.global_params["need_recalc_dataset_params"] = False
#
#         # self.global_params["need_recalc_box_params"] = True
#         self.save_new_param_json()
#         self.delete_roi_vars()
#         # self.global_params["need_recalc_eigen_params"] = True
#         if self.filter_params["pca"]:
#             with open(os.path.join(self.save_dir_path,
#                                    'pca_shape.text'), "w") as f:
#                 f.write("shapes")
#                 f.write(str([x.shape for x in self.pca_decomp if type(x) != bool]))
#     return self.dataset_trials_filtered



from qtpy import QtCore
from qtpy.QtGui import *
from qtpy.QtWidgets import *

from cidan.GUI.ListWidgets.ClassItemModule import ClassItemModule
from cidan.GUI.ListWidgets.ClassItemWidget import ClassItemWidget


class ClassListModule(QFrame):
    """
    Its the class list in the Class modification tab
    """

    def __init__(self, data_handler, classifier_tab, select_multiple=False,
                 display_time=True):
        super().__init__()
        self.current_selected_class = 0
        self.select_multiple = select_multiple
        self.display_time = display_time
        self.classifier_tab = classifier_tab
        self.color_list = data_handler.color_list
        self.list = QListView()

        self.setStyleSheet("QListView::item { border-bottom: 1px solid rgb(50, 65, " +
                           "75); } " + """QListView::item::pressed,
 {
  background-color: #19232D;
  border: 1px solid #32414B;
  color: #F0F0F0;
  gridline-color: #32414B;
  border-radius: 4px;
}""")
        self.top_labels_layout = QHBoxLayout()
        self.top_labels_layout.setContentsMargins(0, 0, 0, 0)
        label1 = QLabel(text="Class")
        label1.setMaximumWidth(170 * ((self.logicalDpiX() / 96.0)))
        self.top_labels_layout.addWidget(label1, alignment=QtCore.Qt.AlignRight)
        label2 = QLabel(text="Class num")
        label2.setMaximumWidth(140 * ((self.logicalDpiX() / 96.0)))
        self.top_labels_layout.addWidget(label2, alignment=QtCore.Qt.AlignLeft)
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)

        self.setMinimumHeight(200 * ((self.logicalDpiX() / 96.0 - 1) / 2 + 1))
        self.model = QStandardItemModel(self.list)
        self.list.setModel(self.model)
        self.layout = QVBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addLayout(self.top_labels_layout)
        self.layout.addWidget(self.list)
        self.class_item_list = []
        self.setLayout(self.layout)

    def keyPressEvent(self, event):
        self.classifier_tab.keyPressEvent(event)

    # def set_current_select(self, num):
    #     self.list.setCurrentIndex(self.model.index(int(num - 1), 0))
    #     self.class_time_check_list[num - 1] = not self.class_time_check_list[num - 1]
    #     self.class_item_list[num - 1].select_check_box()
    #     # self.class_module_list[num-1].
    #     if self.display_time:
    #         self.class_item_list[num - 1].select_time_check_box()

    def set_list_items(self, class_dict):
        self.class_dict = class_dict
        for x in range(self.model.rowCount()):
            self.model.removeRow(0)
        self.class_item_list = []
        self.class_module_list = []
        for num, id in enumerate(list(self.class_dict.keys())):
            item = ClassItemWidget(self.classifier_tab,
                                   self.class_dict[id]["color"], self,
                                   num + 1, name=self.class_dict[id]["name"], id=id
                                   )
            item1 = ClassItemModule(self.class_dict[id]["color"], num + 1,
                                    name=self.class_dict[id]["name"],
                                    id=id, classifier_tab=self.classifier_tab)
            item1.setSelectable(False)
            self.class_item_list.append(item)
            self.model.appendRow(item1)
            self.class_module_list.append(item1)
            self.list.setIndexWidget(item1.index(), item)

        # self.class_item_list[0].select_check_box()
        # self.class_item_list[0].select_time_check_box()
    # def change(self):
    #     # This is a way of running the select class function when a checkbox is clicked there
    #     # needed to be a work around because can't just connect a signal to it
    #     for num, item, check_val in zip(range(1,len(self.class_time_check_list)+1),self.class_item_list,self.class_time_check_list):
    #         if item.checkState() != check_val:
    #             self.class_time_check_list[num-1] = item.checkState()
    #             if item.checkState():
    #                 self.classifier_tab.selectRoi(num)
    #             else:
    #                 self.classifier_tab.deselectRoi(num)



import numpy as np


def mask_data_3d(data, mask):
    from cidan.LSSC.functions.data_manipulation import reshape_to_2d_over_time

    """
    Masks a dataset returning a 2d representation with only relevant pixels

    Parameters
    ----------
    data
        A 3d dataset with x,y time
    mask
        A list of pixels to mask in a 1d representation of the pixels
    Returns
    -------
        A 2d data format with pixel num and time as axis
    """
    if len(mask.shape) == 2:
        mask = mask.flatten()
    data_2d = reshape_to_2d_over_time(data)
    data_2d_masked = data_2d[mask]
    return data_2d_masked


def mask_data_2d(data, mask):
    """
    Masks a dataset returning a 2d representation with only relevant pixels

    Parameters
    ----------
    data
        A 2d dataset with pixel num, time
    mask
        A list of pixels to mask in a 1d representation of the pixels
    Returns
    -------
        A 2d data format with pixel num and time as axis
    """
    if len(mask.shape) == 2:
        mask = mask.flatten()
    data_2d_masked = data[mask]
    return data_2d_masked


def mask_to_data_2d(data_masked, mask):
    """
    Takes a masked data and transforms it back into the original
    Parameters
    ----------
    data_masked 1d or 2d data to reformat
    mask 1d mask
    original_shape shape of the 2d data

    Returns
    -------

    """
    if len(mask.shape) == 2:
        mask = mask.flatten()
    if len(data_masked.shape) == 1:
        data_masked = data_masked.reshape((-1, 1))
    num_rows = data_masked.shape[1]
    data_all = np.zeros((mask.shape[0], num_rows))
    data_all = data_all.reshape((-1))
    data_masked = data_masked.reshape((-1))
    mask_two = mask.reshape((-1)).repeat(num_rows)
    data_all[mask_two] = data_masked
    data_all = data_all.reshape((mask.shape[0], num_rows))
    return data_all


def mask_to_data_point(points_list_masked, mask, original_shape):
    """
    Takes a list of points in the masked dataset and transforms them to point numbers in
     original dataset
    Parameters
    ----------
    points_list_masked
    mask
    original_shape

    Returns
    -------

    """
    if len(mask.shape) == 2:
        mask = mask.flatten()
    mask_sum = np.cumsum(mask)

    def get_index(point_num):
        return np.nonzero(mask_sum == point_num + 1)[0][0]

    return [get_index(point_num=point) for point in points_list_masked]


def orig_to_mask_data_point(points_list, mask, original_shape):
    """
    Takes a list of points in the masked dataset and transforms them to point numbers in
     original dataset
    Parameters
    ----------
    points_list_masked
    mask
    original_shape

    Returns
    -------

    """
    if len(mask.shape) == 2:
        mask = mask.flatten()
    mask_sum = np.cumsum(mask)

    return mask_sum[points_list] - 1
    # return [mask_sum[point]-1 for point in points_list_masked]



import numpy as np


def calculateMeanTrace(roi_data, neuropil_data,
                       sub_neuropil=False):
    """
    Calcualtes the mean time trace for a given roi and data
    Parameters
    ----------
    roi list of pixels for data (1D cords)
    data 2D list of data for a single trials with pixel num as dim 0, time as dim 1

    Returns
    -------
    A 1d nd array with the time trace for the ROI
    """

    mean = np.mean(roi_data, axis=0)
    if sub_neuropil:
        mean = mean - np.mean(neuropil_data, axis=0)
    # if denoise:
    #     mean = waveletDenoise(mean)
    return mean


def neuropil(roi_data, neuropil_data, ):
    return np.mean(neuropil_data, axis=0)



import numpy as np

from cidan.LSSC.functions.data_manipulation import load_filter_tif_stack


def test_load_filter_tif_stack():
    test, stack = load_filter_tif_stack(path="test_files/small_dataset1.tif",
                                        filter=True,
                                        median_filter=True, median_filter_size=[3, 3, 3],
                                        z_score=False, slice_stack=False, slice_start=0,
                                        slice_every=3, crop_stack=False, crop_x=[0, 400],
                                        crop_y=[0, 155])
    assert stack.shape == (99, 400, 150)
    assert stack.dtype == np.float32
    test, stack = load_filter_tif_stack(path="test_files/small_dataset1.tif",
                                        filter=True,
                                        median_filter=False, median_filter_size=[3, 3, 3],
                                        z_score=True, slice_stack=True, slice_start=0,
                                        slice_every=3, crop_stack=True, crop_x=[4, 200],
                                        crop_y=[4, 125])
    assert stack.shape == (33, 196, 121)
    assert stack.dtype == np.float32
    mean = np.mean(stack, axis=0)
    assert np.max(mean) < .01
    assert np.min(mean) > -.01



import dask
from qtpy import QtCore
from qtpy.QtWidgets import *

from cidan.GUI.Data_Interaction.PreprocessThread import PreprocessThread
from cidan.GUI.Inputs.OptionInput import OptionInput
from cidan.GUI.SettingWidget.SettingsModule import preprocessing_settings
from cidan.GUI.Tabs.Tab import Tab


class PreprocessingTab(Tab):
    """Class controlling the Preprocessing tab, inherits from Tab


    Attributes
    ----------
    main_widget : MainWidget
        A reference to the main widget
    data_handler : DataHandler
        A reference to the main DataHandler of MainWidget

    """

    def __init__(self, main_widget):
        self.main_widget = main_widget
        self.image_view = self.main_widget.preprocess_image_view
        # This part initializes the button to process the data
        process_button = QPushButton()
        process_button.setText("Apply Settings")
        process_button_layout = QVBoxLayout()
        process_button_widget = QWidget()
        process_button_widget.setLayout(process_button_layout)
        process_button_layout.addWidget(process_button)
        process_button_layout.setContentsMargins(0, 0, 0, 0)
        thread = PreprocessThread(main_widget, process_button, self)
        main_widget.thread_list.append(thread)  # Appends the thread to the main
        # widget thread list
        process_button.clicked.connect(lambda: thread.runThread())

        # Section that creates all the buttons to change which image is displayed
        image_buttons = QWidget()
        self._image_buttons_layout = QHBoxLayout()
        self._image_buttons_layout.setContentsMargins(2, 0, 2, 0)
        image_buttons.setLayout(self._image_buttons_layout)
        self.max_image_button = QPushButton()
        self.max_image_button.setText("Max Image")
        self.max_image_button.clicked.connect(
            lambda: self.set_image_display_list(self.data_handler.trials_loaded,
                                                self.data_handler.max_images,
                                                "Max Image", self.max_image_button))
        stack_button = QPushButton()
        stack_button.setText("Filtered Stack")
        stack_button.clicked.connect(
            lambda: self.set_image_display_list(self.data_handler.trials_loaded,
                                                self.data_handler.dataset_trials_filtered_loaded,
                                                "Filtered Stack", stack_button))
        self.pca_stack_button = QPushButton()
        self.pca_stack_button.setText("PCA Stack")
        self.pca_stack_button.clicked.connect(
            lambda: self.set_image_display_list(self.data_handler.trials_loaded,
                                                self.data_handler.pca_decomp,
                                                "PCA Stack", self.pca_stack_button))
        self.mean_image_button = QPushButton()
        self.mean_image_button.setText("Mean Image")
        self.mean_image_button.clicked.connect(
            lambda: self.set_image_display_list(self.data_handler.trials_loaded,
                                                self.data_handler.mean_images,
                                                "Mean Image", self.mean_image_button))
        self.temporal_correlation_image_button = QPushButton()
        self.temporal_correlation_image_button.setText("Temporal Correlation Image")
        self.temporal_correlation_image_button.clicked.connect(
            lambda: self.set_image_display_list(self.data_handler.trials_loaded,
                                                self.data_handler.temporal_correlation_images,
                                                "Temporal Correlation Image",
                                                self.temporal_correlation_image_button))
        self._image_buttons_layout.addWidget(stack_button)
        self._image_buttons_layout.addWidget(self.max_image_button)
        self._image_buttons_layout.addWidget(self.mean_image_button)
        # self._image_buttons_layout.addWidget(self.temporal_correlation_image_button)
        self._image_buttons_layout.addWidget(self.pca_stack_button)

        main_widget.preprocess_image_view.setContentsMargins(0, 0, 0, 0)
        # main_widget.preprocess_image_view.setMargin(0)
        preprocessing_settings_widget = preprocessing_settings(main_widget)
        preprocessing_settings_widget.setContentsMargins(0, 0, 0, 0)
        # preprocessing_settings_widget.setMaximumWidth(400)
        # preprocessing_settings_widget.setMinimumWidth(int(400*((self.logicalDpiX() / 96.0-1)/2+1)))
        # main_widget.preprocess_image_view.setMinimumWidth(500)
        # Update image view
        self.updateTab()
        main_widget.preprocess_image_view.setSizePolicy(QSizePolicy.Expanding,
                                                        QSizePolicy.Expanding)
        # Initialize the tab with the necessary columns
        super().__init__("Preprocessing", column_1=[preprocessing_settings_widget
                                                    ],
                         column_2=[main_widget.preprocess_image_view
                                   ], horiz_moveable=True)
        # process_button_widget.setMaximumWidth(400*((self.logicalDpiX() / 96.0-1)/2+1)

        self.layout.setContentsMargins(0, 0, 0, 0)
        self.column_1_layout.addWidget(process_button_widget,
                                       alignment=QtCore.Qt.AlignBottom)
        self.column_1_layout.setContentsMargins(0, 0, 0, 0)
        self.column_2_layout.addWidget(image_buttons, alignment=QtCore.Qt.AlignBottom)
        self.column_2_layout.setContentsMargins(0, 0, 0, 0)

    @property
    def data_handler(self):
        return self.main_widget.data_handler

    def set_image_display_list(self, trial_names, data_list, name, button=None):
        """
        Sets the preprocessing image display to use an option input and set data list
        Parameters
        ----------
        trial_names : List[str]
            the names of each trial
        data_list : List[np.ndarray]
            Corresponding data for each trial
        Returns
        -------
        Nothing
        """
        if hasattr(self.data_handler, "total_size"):
            total_size = (
                str(self.data_handler.total_size[0]),
                str(self.data_handler.total_size[1]))
        else:
            total_size = [0, 0]
        if len(data_list[0].shape) == 3:
            cur_size = [str(data_list[0].shape[1]), str(data_list[0].shape[2])]
        else:
            cur_size = [str(data_list[0].shape[0]), str(data_list[0].shape[1])]
        def set_image(x, trial_name):
            def end_func():
                try:
                    self.main_widget.preprocess_image_view.setImage(
                        data_list[trial_names.index(trial_name)][:])
                    self.main_widget.console.updateText(
                        "Loaded filtered trial %s" % str(trial_name))
                    self.image_view.image_label.setText(
                        "Trial: %s, " % str(
                            trial_name) + name + ", Original Size: (%s, %s), Cropped Size: (%s, %s)" % (
                            total_size[1], total_size[0], cur_size[1], cur_size[0]))
                    self.current_trial_name = trial_name
                except TypeError:
                    self.updateTab()

            if (self.main_widget.checkThreadRunning()):
                try:
                    if type(data_list[trial_names.index(trial_name)]) == bool or \
                            type(self.data_handler.dataset_trials_filtered_loaded[
                                     trial_names.index(trial_name)]) == type(
                        dask.delayed(min)()):
                        self.main_widget.console.updateText(
                            "Applying filters to trial %s" % str(trial_name))
                        self.main_widget.calculate_single_trial_thread.runThread(
                            trial_names.index(trial_name), end_func)
                    else:
                        self.main_widget.preprocess_image_view.setImage(
                            data_list[trial_names.index(trial_name)][:])
                        self.main_widget.console.updateText(
                            "Loaded filtered trial %s" % str(trial_name))
                        self.image_view.image_label.setText(
                            "Trial: %s, " % str(
                                trial_name) + name + ", Original Size: (%s, %s), Cropped Size: (%s, %s)" % (
                                total_size[1], total_size[0], cur_size[1], cur_size[0]))
                        self.current_trial_name = trial_name
                except TypeError:
                    self.updateTab()
            elif self.trial_selector_input.current_state() != self.current_trial_name:
                self.trial_selector_input.set_val(self.current_trial_name)
                print("Please wait until current process is done")

        if (self.main_widget.checkThreadRunning()):
            if hasattr(self, "trial_selector_input"):
                self.trial_selector_input.setParent(None)
            if hasattr(self, "trial_selector_input"):
                current = self.trial_selector_input.input_box.currentIndex()
            else:
                current = 0
            # if button != None:
            #     button.setStyleSheet("QPushButton {border 1px solid #148CD2; }")
            self.trial_selector_input = OptionInput("", "", set_image,
                                                    val_list=trial_names,
                                                    tool_tip="Select time block to display",
                                                    display_tool_tip=False,
                                                    default_index=0,
                                                    show_name=False)
            self.trial_selector_input.input_box.setCurrentIndex(current)
            self._image_buttons_layout.addWidget(self.trial_selector_input)
            self.current_trial_name = trial_names[current]
            set_image("", trial_names[current])

            self.main_widget.console.updateText(
                "Setting current background to: %s" % name)
        else:
            print("Please wait until current process is done")
    def set_image_display(self, data):
        self.trial_selector_input.setParent(None)
        self.trial_selector_input = OptionInput("", "", lambda x, y: 3,
                                                val_list=["All"],
                                                tool_tip="Select Timeblock to display",
                                                display_tool_tip=False, default_index=0,
                                                show_name=False)
        self._image_buttons_layout.addWidget(self.trial_selector_input)
        self.main_widget.preprocess_image_view.setImage(
            data.copy())

    def updateTab(self):
        if (self.main_widget.checkThreadRunning()):
            self.pca_stack_button.setEnabled(self.data_handler.filter_params["pca"])
            self.set_image_display_list(self.data_handler.trials_loaded,
                                        self.data_handler.max_images, "Max Image")



import os
import signal

from PySide2.QtCore import QThreadPool
from qtpy import QtGui
from regional import many

from cidan.GUI.Data_Interaction.CalculateSingleTrialThread import \
    CalculateSingleTrialThread
from cidan.GUI.Data_Interaction.DemoDownloadThread import DemoDownloadThread
from cidan.GUI.Data_Interaction.OpenDatasetThread import OpenDatasetThread
from cidan.GUI.EventFilter.EventFilterMainWindow import EventFilterMainWindow
from cidan.GUI.Inputs.FloatInput import FloatInput
from cidan.GUI.Tabs.AnalysisTab import AnalysisTab
from cidan.GUI.Tabs.ClassificationTab import ClassificationTab
from cidan.LSSC.SpatialBox import SpatialBox

os.environ['QT_MAC_WANTS_LAYER'] = '1'
os.environ['QT_API'] = 'pyside2'
from qtpy.QtWidgets import QTabWidget
from cidan.GUI.Tabs.FileOpenTab import FileOpenTab, createFileDialog
from cidan.GUI.Tabs.ROIExtractionTab import *
from cidan.GUI.Tabs.PreprocessingTab import *
import qdarkstyle
from cidan.GUI.ImageView.ImageViewModule import ImageViewModule
from cidan.GUI.Data_Interaction.DataHandler import DataHandler
from cidan.GUI.Console.ConsoleWidget import ConsoleWidget
import sys
import logging
import pyqtgraph as pg
import cidan.GUI.Data_Interaction.neurofinder_edited

# sys._excepthook = sys.excepthook
# def exception_hook(exctype, value, traceback):
#     print(exctype, value, traceback)
#     sys._excepthook(exctype, value, traceback)
#     sys.exit(1)
# sys.excepthook = exception_hook

pg.setConfigOption("imageAxisOrder", "row-major")
def SIGSEGV_signal_arises(signalNum, stack):
    print(f"{signalNum} : SIGSEGV arises")
    # Your code

signal.signal(signal.SIGSEGV, SIGSEGV_signal_arises)
class MainWindow(QMainWindow):
    """Initializes the basic window with Main widget being the focused widget"""

    def __init__(self, dev=False, preload=False, widefield=False):
        super().__init__()
        self.title = 'cidan'
        scale = (self.logicalDpiX() / 96.0 - 1) / 2 + 1
        sizeObject = QtGui.QGuiApplication.primaryScreen().availableGeometry()

        self.width = 1500 * scale
        self.height = 1066.6 * scale
        if self.height > sizeObject.height() * .95:
            self.height = sizeObject.height() * .95
        if self.width > sizeObject.width() * .95:
            self.width = sizeObject.width() * .95
        self.setWindowTitle(self.title)
        self.setMinimumSize(int(self.width), int(self.height))
        self.main_menu = self.menuBar()
        self.setContentsMargins(0, 0, 0, 0)
        self.widefield = widefield

        import cidan
        cidanpath = os.path.dirname(os.path.realpath(cidan.__file__))
        print(cidanpath)
        icon_path = os.path.join(
            cidanpath, "logo", "logo.png"
        )

        app_icon = QtGui.QIcon()
        app_icon.addFile(icon_path, QtCore.QSize(16, 16))
        app_icon.addFile(icon_path, QtCore.QSize(24, 24))
        app_icon.addFile(icon_path, QtCore.QSize(32, 32))
        app_icon.addFile(icon_path, QtCore.QSize(48, 48))
        app_icon.addFile(icon_path, QtCore.QSize(96, 96))
        app_icon.addFile(icon_path, QtCore.QSize(256, 256))
        self.setWindowIcon(app_icon)

        self.table_widget = MainWidget(self, dev=dev, preload=preload,
                                       widefield=widefield)
        self.setCentralWidget(self.table_widget)
        # self.setStyleSheet(qdarkstyle.load_stylesheet())
        style = str("""
            
            QWidget {font-size: %dpx;}
            QTabWidget {font-size: %dpx; padding:0px; margin:%dpx;
                border:0px;}
            QTabBar::tab {
                background: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                           stop: 0 #E1E1E1, stop: 0.4 #DDDDDD,
                                          stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);
                /*border: 2px solid #C4C4C3;*/
                /*border-bottom-color: #C2C7CB; !* same as the pane color *!*/
                
                min-width: 8ex;
                padding:%dpx;
                border:%dpx;
            }
            
            QComboBox::item:checked {
              font-weight: bold;
              height: %dpx;
            }
            """ % (
            20 * scale, 20 * scale, 0 * scale, 0 * scale, 0 * scale, 20 * scale))
        self.setStyleSheet(qdarkstyle.load_stylesheet() + style)

        # extractAction.triggered.connect()

        self.show()


class MainWidget(QWidget):
    """Main Widget, contains everything

    Attributes
    ----------
    main_window : MainWindow
        A reference to the main window of the application
    main_menu : ???
        the top bar menu
    layout : QLayout
        The main layout for the widget
    data_handler : DataHandler
        The instance that controls all interactions with dataset
    thread_list : List[Thread]
        A list of all the possible running threads, used to ensure only 1 thread is
        running at a time
    preprocess_image_view : ImageViewModule
        The image view for the preprocess tab
    roi_image_view : ImageViewModule
        The image view for the roi extraction tab
    tab_widget : QTabWidget
        Controls the main tabs of the application
    console : ConsoleWidget
        Widget for the console
    tabs : List[Tabs]
        A list of the currently active tabs not used until after init_w_data is run
    """

    def __init__(self, parent, dev=False, preload=True, widefield=False):
        """
        Initialize the main widget to load files
        Parameters
        ----------
        parent
        """
        super().__init__(parent)
        self.scale = (self.logicalDpiX() / 96.0 - 1) / 2 + 1
        self.main_window = parent
        self.threadpool = QThreadPool()
        self.progress_signal = None
        self.main_menu = self.main_window.main_menu
        self.layout = QVBoxLayout(self)
        self.eventFilterCustom = EventFilterMainWindow(self)

        self.data_handler = None
        self.thread_list = []
        self.setContentsMargins(0, 0, 0, 0)

        self.dev = dev
        self.widefield = widefield
        self.tab_widget = QTabWidget()
        self.tab_widget.setContentsMargins(0, 0, 0, 0)
        self.fileOpenTab = FileOpenTab(self)
        self.tab_widget.addTab(self.fileOpenTab, "Open Dataset")
        # This part add placeholder tabs until data is loaded
        self.tabs = ["Preprocessing", "ROI Extraction", "Analysis"]
        for num, tab in enumerate(self.tabs):
            self.tab_widget.addTab(QWidget(), tab)
            self.tab_widget.setTabEnabled(num + 1, False)
        self.layout.addWidget(self.tab_widget)
        #
        self.console = ConsoleWidget(self)
        self.console.setContentsMargins(0, 0, 0, 0)
        # self.console.setMaximumHeight(150)
        # self.console.setMinimumHeight(150)
        self.layout.addWidget(self.console)
        self.setLayout(self.layout)
        self.installEventFilter(self.eventFilterCustom)
        self.console.installEventFilter(self.eventFilterCustom)
        self.tab_widget.installEventFilter(self.eventFilterCustom)

        self.demo_download_thread = DemoDownloadThread(main_widget=self)
        self.thread_list.append(self.demo_download_thread)
        self.open_dataset_thread = OpenDatasetThread(main_widget=self)
        self.thread_list.append(self.open_dataset_thread)
        self.calculate_single_trial_thread = CalculateSingleTrialThread(self)
        self.thread_list.append(self.calculate_single_trial_thread)

        # Initialize top bar menu
        self.fileMenu = self.main_menu.addMenu('&File')
        openFileAction = QAction("Open File", self)
        openFileAction.setStatusTip('Open a single file')
        openFileAction.triggered.connect(lambda: self.selectOpenFileTab(0))
        self.fileMenu.addAction(openFileAction)
        openFolderAction = QAction("Open Folder", self)
        openFolderAction.setStatusTip('Open a folder')
        openFolderAction.triggered.connect(lambda: self.selectOpenFileTab(1))
        self.fileMenu.addAction(openFolderAction)
        openPrevAction = QAction("Open Previous Session", self)
        openPrevAction.setStatusTip('Open a previous session')
        openPrevAction.triggered.connect(lambda: self.selectOpenFileTab(2))
        self.fileMenu.addAction(openPrevAction)
        openPrevAction = QAction("Download and Open Demo Dataset", self)
        openPrevAction.setStatusTip('Download and Open Demo Dataset')
        openPrevAction.triggered.connect(lambda: self.downloadOpenDemo())
        self.fileMenu.addAction(openPrevAction)
        # Below here in this function is just code for testing
        # TODO check if it can load data twice
        if preload and dev:
            try:
                # auto loads a small dataset
                self.data_handler = DataHandler(

                    "/Users/sschickler/Code_Devel/LSSC-python/tests/test_files/",
                    "/Users/sschickler/Code_Devel/LSSC-python/tests/test_files/save_dir",
                    trials=["small_dataset1.tif", "small_dataset2.tif"],
                    save_dir_already_created=True, load_into_mem=False,
                    auto_crop=False)
                # self.data_handler.calculate_filters(auto_crop=True)
                self.init_w_data()
            except IndentationError:
                pass
        if False and dev:
            # auto loads a large dataset
            self.data_handler = DataHandler(
                "/Users/sschickler/Code Devel/LSSC-python/input_images/dataset_1",
                "/Users/sschickler/Code Devel/LSSC-python/input_images/test3",
                save_dir_already_created=False)
            self.init_w_data()

    def init_w_data(self):
        """
        Initialize main widget with data

        It activates the other tabs and helps load the data into image views
        Returns
        -------

        """
        self.thread_list = []
        self.demo_download_thread = DemoDownloadThread(main_widget=self)
        self.thread_list.append(self.demo_download_thread)
        self.open_dataset_thread = OpenDatasetThread(main_widget=self)
        self.thread_list.append(self.open_dataset_thread)
        self.calculate_single_trial_thread = CalculateSingleTrialThread(self)
        self.thread_list.append(self.calculate_single_trial_thread)

        self.preprocess_image_view = ImageViewModule(self)
        # This assumes that the data is already loaded in
        for num, _ in enumerate(self.tabs):
            self.tab_widget.removeTab(1)
        self.roi_extraction_tab = ROIExtractionTab(self)
        # TODO add to export tab to export all time traces or just currently caclulated ones
        self.tabs = [PreprocessingTab(self), self.roi_extraction_tab, AnalysisTab(self)]
        if self.dev:
            self.tabs.insert(2, ClassificationTab(self))
        # Add tabs
        for tab in self.tabs:
            self.tab_widget.addTab(tab, tab.name)
        self.tab_widget.setCurrentIndex(1)
        self.image_view_list = [x.image_view for x in self.tabs]
        # self.tab_widget.currentChanged.connect(
        #     lambda x: self.tabs[1].image_view.set_background("",
        #                                                      self.tabs[
        #                                                          1].image_view.background_chooser.current_state(),
        #                                                      update_image=True))
        # self.tab_widget.currentChanged.connect(
        #     lambda x: self.tabs[2].image_view.reset_view())
        # self.tab_widget.currentChanged.connect(
        #     lambda x: self.tabs[2].reset_view())
        if not hasattr(self, "export_menu"):
            self.export_menu = self.main_menu.addMenu("&Export")
            export_action = QAction("Export Time Traces/ROIs", self)
            export_action.setStatusTip('Export Time Traces/ROIs')
            export_action.triggered.connect(lambda: self.exportStuff())
            self.export_menu.addAction(export_action)
        if not hasattr(self, "tool_menu"):
            self.tool_menu = self.main_menu.addMenu("&Tools")
        if not hasattr(self, "import_json"):
            self.import_json = QAction("Import Json", self)
            self.import_json.setStatusTip('Import Json')
            self.import_json.triggered.connect(lambda: self.openNewJSON())
            self.tool_menu.addAction(self.import_json)
        if not hasattr(self, "auto_label_rois") and self.dev:
            self.auto_label_rois = QAction("Auto Label ROIS", self)
            self.auto_label_rois.setStatusTip('Use a json file to auto label rois')
            self.auto_label_rois.triggered.connect(lambda: self.autoLabelRois())
            self.tool_menu.addAction(self.auto_label_rois)
        if not hasattr(self, "merge_mode_item") and self.dev:
            self.merge_mode_item = QAction("Turn merge mode on", self)
            self.merge_mode_item.triggered.connect(lambda: self.merge_mode_func())
            self.tool_menu.addAction(self.merge_mode_item)
        # self.autoLabelRois()
    def selectOpenFileTab(self, index):
        self.tab_widget.setCurrentIndex(0)
        self.fileOpenTab.tab_selector.setCurrentIndex(index)

    def merge_mode_func(self):
        self.roi_extraction_tab.merge_mode = not self.roi_extraction_tab.merge_mode
        cur_val = self.roi_extraction_tab.merge_mode
        if cur_val:
            self.merge_mode_item.setText("Turn merge mode off")
        else:
            self.merge_mode_item.setText("Turn merge mode on")
        self.roi_extraction_tab.toggle_merge_mode(cur_val)

    def openNewJSON(self):
        import json
        path = createFileDialog(directory="~/Desktop", forOpen=True,
                                isFolder=False, name="Select Json")
        with open(path, "r") as f:
            # test2=f.read()
            test = json.loads(f.read())
        try:
            print("Importing json")
            if self.data_handler.dataset_params["crop_stack"]:
                rois = [[(y[0] - self.data_handler.dataset_params["crop_x"][0],
                          y[1] - self.data_handler.dataset_params["crop_y"][0]) for y in
                         x["coordinates"]] for x in test]
            else:
                rois = [[(y[0], y[1]) for y in x["coordinates"]] for x in test]

            from cidan.LSSC.functions.data_manipulation import cord_2d_to_pixel_num
            rois = [
                cord_2d_to_pixel_num(np.array(roi).transpose(), self.data_handler.shape)
                for roi in rois]
            self.data_handler.rois_dict = {(num + 1): {"pixels": roi, "index": num} for
                                           num, roi
                                           in enumerate(rois)}
            self.data_handler.save_rois(self.data_handler.rois)

            self.data_handler.gen_roi_display_variables()

            def temp():
                self.updateTabs()
                self.console.updateText("Finished json import")

            [x for x in self.thread_list if type(x) == TimeTraceCalculateThread][
                0].runThread(temp)
        except:
            print(
                "Error importing json, please make sure json is valid and crop is correct")

    def autoLabelRois(self):
        import json
        path = "/Users/sschickler/Code_Devel/nuerofinder/neurofinder.00.04/regions/regions.json"
        #createFileDialog(directory="~/Desktop", forOpen=True,
                #                isFolder=False, name="Select Json")
        if "true" in list(self.data_handler.classes.keys()):
            for roi in self.data_handler.classes["true"]["rois"]:
                self.data_handler.delete_roi(roi, input_key=True, update_display=False)

        self.data_handler.classes = {"Unassigned": {"color": (150, 150, 150), "rois": [],
                                               "name": "Unassigned", "editable": False},
                                "Soma": {"color": self.data_handler.color_list[0], "rois": [],
                                         "name": "Soma", "editable": True},
                                "Dendrite": {"color": self.data_handler.color_list[1], "rois": [],
                                              "name": "Dendrite", "editable": True},
                                 "Background": {"color": self.data_handler.color_list[2], "rois": [],
                                              "name": "Background", "editable": True},
                                     "Merged": {"color": self.data_handler.color_list[3], "rois": [],
                                                  "name": "Merged", "editable": True},
                                     "Partial": {"color": self.data_handler.color_list[4], "rois": [],
                                                  "name": "Partial", "editable": True},
                                     "Some/Background": {"color": self.data_handler.color_list[5], "rois": [],
                                                 "name": "Soma/Background", "editable": True},
                                     "Some/Dendrite": {"color": self.data_handler.color_list[6], "rois": [],
                                                         "name": "Soma/Dendrite", "editable": True},
                                    "Apical Dendrite": {"color": self.data_handler.color_list[6], "rois": [],
                                                       "name": "Apical Dendrite", "editable": True}
                                    }
        with open(path, "r") as f:
            # test2=f.read()
            test = json.loads(f.read())

        try:
            print("Importing json")

            from cidan.LSSC.functions.data_manipulation import cord_2d_to_pixel_num

            if self.data_handler.dataset_params["crop_stack"]:
                rois = [[(y[0] if y[0] <self.data_handler.dataset_params["crop_x"][1]-1 else self.data_handler.dataset_params["crop_x"][1]-1, y[1] if y[1] <self.data_handler.dataset_params["crop_y"][1]-1 else self.data_handler.dataset_params["crop_y"][1]-1) for y in x["coordinates"]] for x in test]
                rois = [[(y[0] if y[0] >self.data_handler.dataset_params["crop_x"][0] else self.data_handler.dataset_params["crop_x"][0], y[1] if y[1] >self.data_handler.dataset_params["crop_y"][0] else self.data_handler.dataset_params["crop_y"][0]) for y in x] for x in rois]

                rois = [[(y[0] - self.data_handler.dataset_params["crop_x"][0],
                          y[1] - self.data_handler.dataset_params["crop_y"][0]) for y in
                         x] for x in rois]
            else:
                rois = [[(y[0], y[1]) for y in x["coordinates"]] for x in test]

            from cidan.LSSC.functions.data_manipulation import cord_2d_to_pixel_num
            rois_true_1d = [
                cord_2d_to_pixel_num(np.array(roi).transpose(), self.data_handler.shape)
                for roi in rois]
            spatial_box = SpatialBox(0, 1, image_shape=self.data_handler.shape,
                                     spatial_overlap=0)
            rois_true = many(rois)

            rois = many(
                [spatial_box.convert_1d_to_2d(roi) for roi in self.data_handler.rois])
            matches = cidan.GUI.Data_Interaction.neurofinder_edited.match(rois,
                                                                          rois_true,
                                                                          threshold=5)
            dialog = QDialog()
            dialog.setStyleSheet(qdarkstyle.load_stylesheet())
            dialog.layout = QVBoxLayout()
            overlap_1_input = FloatInput(display_name="Overlap/current threshold:",
                                         default_val=0.7, max=10, min=0,
                                         on_change_function=None, program_name="test",
                                         step=.01, tool_tip=None)
            overlap_2_input = FloatInput(display_name="Overlap/true threshold:",
                                         default_val=0.7, max=10, min=0,
                                         on_change_function=None, program_name="test",
                                         step=.01, tool_tip=None)
            dialog.layout.addWidget(overlap_1_input, alignment=QtCore.Qt.AlignCenter)
            dialog.layout.addWidget(overlap_2_input, alignment=QtCore.Qt.AlignCenter)

            def final_load_part():
                dialog.close()
                overlap_1_thresh = overlap_1_input.current_state()
                overlap_2_thresh = overlap_2_input.current_state()

                for key in self.data_handler.classes.keys():
                    self.data_handler.classes[key]["rois"] = []
                for num, match in enumerate(matches):
                    if not np.isnan(match):
                        overlap = float(np.intersect1d(self.data_handler.rois[num],
                                                       rois_true_1d[match]).size)
                        overlap_1 = overlap / len(self.data_handler.rois[num])
                        overlap_2 = overlap / len(rois_true_1d[match])
                        if overlap_1 >= overlap_1_thresh and overlap_2 >= overlap_2_thresh:
                            self.data_handler.classes[
                                list(self.data_handler.classes.keys())[1]][
                                'rois'].append(
                                self.data_handler.roi_index_backward[num])
                        else:
                            self.data_handler.classes[
                                list(self.data_handler.classes.keys())[5]][
                                'rois'].append(
                                self.data_handler.roi_index_backward[num])
                    else:
                        self.data_handler.classes[
                            list(self.data_handler.classes.keys())[3]]['rois'].append(
                            self.data_handler.roi_index_backward[num])

                self.data_handler.classes["true"] = {

                    "color": DataHandler._color_list[3], "rois": [],
                    "name": "True", "editable": True}
                for x in rois_true_1d:
                    self.data_handler.classes["true"]["rois"].append(
                        self.data_handler.add_new_roi(x, update=False))
                self.data_handler.gen_roi_display_variables()
                self.data_handler.rois_update_needed = True
                self.data_handler.rois  # just to force the update
                self.data_handler.gen_class_display_variables()
                self.updateTabs()

            load_button = QPushButton(text="Auto Label ROIs")
            load_button.clicked.connect(lambda x: final_load_part())
            dialog.layout.addWidget(load_button, alignment=QtCore.Qt.AlignCenter)
            dialog.setLayout(dialog.layout)
            dialog.show()
        except:
            print(
                "Error importing json, please make sure json is valid")

    def exportStuff(self):
        dialog = QDialog()
        dialog.setStyleSheet(qdarkstyle.load_stylesheet())
        dialog.layout = QVBoxLayout()

        trial_dialog = TrialListWidget()
        trial_dialog.set_items_from_list(self.data_handler.trials_all,
                                         trials_selected_indices=self.data_handler.trials_loaded_time_trace_indices)
        settings_layout = QHBoxLayout()
        style = str("""
                    QVBoxLayout {
                
                        border:%dpx;
                    }


                    """ % (2))
        dialog.setStyleSheet(qdarkstyle.load_stylesheet() + style)
        image_selection_layout = QVBoxLayout()
        image_selection_layout.addWidget(QLabel("Select background images to use:"),
                                         alignment=QtCore.Qt.AlignTop)

        image_selection_buttons = []
        for button_name in ["Max Image", "Mean Image", "Eigen Norm Image", "Blank"]:
            temp = QCheckBox(button_name)
            temp.toggle()
            image_selection_layout.addWidget(temp, alignment=QtCore.Qt.AlignTop)
            image_selection_buttons.append(temp)
        color_map_selection_layout = QVBoxLayout()
        color_map_selection_layout.addWidget(QLabel("Select color maps to use:"),
                                             alignment=QtCore.Qt.AlignTop)
        color_map_buttons = []
        for button_name in ["Grey Scale", "Green Scale", "Magma", "Virdis", "Plasma",
                            "Cividis", "Hot"]:
            temp = QCheckBox(button_name)
            color_map_selection_layout.addWidget(temp, alignment=QtCore.Qt.AlignTop)
            color_map_buttons.append(temp)
        color_map_buttons[0].toggle()
        settings_layout.addLayout(image_selection_layout, alignment=QtCore.Qt.AlignTop)
        settings_layout.addLayout(color_map_selection_layout,
                                  alignment=QtCore.Qt.AlignTop)
        export_matlab = QCheckBox("Export Matlab files")
        export_matlab.toggle()

        def export_func():
            dialog.close()
            self.data_handler.update_selected_trials(
                trial_dialog.selectedTrials())
            self.data_handler.export(matlab=export_matlab.isChecked(),
                                     background_images=[x for num, x in enumerate(
                                         ["max", "mean", "eigen_norm", "blank"]) if
                                                        color_map_buttons[
                                                            num].isChecked()],
                                     color_maps=[x for num, x in enumerate(
                                         ["gray", "green", "magma", "plasma", "cividis",
                                          "hot"]) if color_map_buttons[num].isChecked()]
                                     )

            msg = QMessageBox()
            msg.setStyleSheet(qdarkstyle.load_stylesheet())
            msg.setWindowTitle("Export data")

            msg.setText("Data Exported to save directory: "+  self.data_handler.save_dir_path)
            msg.setIcon(QMessageBox.Information)
            x = msg.exec_()

        if self.data_handler.real_trials:
            dialog.setWindowTitle("Select Trials to Export")
            dialog.layout.addWidget(trial_dialog)
        else:
            dialog.setWindowTitle("Export:")
        export_button = QPushButton("Export")
        export_button.clicked.connect(lambda x: export_func())
        dialog.layout.addLayout(settings_layout, alignment=QtCore.Qt.AlignCenter)
        dialog.layout.addWidget(export_matlab, alignment=QtCore.Qt.AlignCenter)
        dialog.layout.addWidget(export_button, alignment=QtCore.Qt.AlignCenter)
        dialog.setLayout(dialog.layout)
        dialog.show()

    def checkThreadRunning(self):
        if (any([x.isRunning() for x in self.thread_list])):
            msg = QMessageBox()
            msg.setStyleSheet(qdarkstyle.load_stylesheet())
            msg.setWindowTitle("Operation Denied")

            msg.setText(
                "Sorry we can't preform this operation until current process is "
                "finished")
            msg.setIcon(QMessageBox.Information)
            x = msg.exec_()
            return False

        else:
            return True

    # def quitAllThreads(self):
    #     print("test")
    #
    #     for x in self.thread_list:
    #         if x.isRunning():
    #             print("killing thread")
    #             x.quit()
    #             x.wait()
    def updateTabs(self):
        for x in self.tabs:
            x.updateTab()

    def downloadOpenDemo(self):
        def endfunc(success):
            if success:
                self.console.updateText("Finished Downloading, now processing")
                path_full = os.path.join(path, "CIDAN_Demo/")

                self.data_handler = DataHandler(

                    path_full,
                    path_full,
                    trials=["demo_dataset_1.tif"],
                    save_dir_already_created=False, load_into_mem=True)
                self.init_w_data()
            else:
                self.console.updateText("Download Unsuccessful")
        path = createFileDialog(directory="~/Desktop", forOpen=False,
                                isFolder=True,
                                name="Choose location to save demo dataset")
        self.demo_download_thread.runThread(path, endfunc)
        # self.console.updateText("Downloading Demo Dataset to: " + path)



if __name__ == "__main__":
    # client = Client(processes=False, threads_per_worker=16,
    #                 n_workers=1, memory_limit='32GB')
    # print(client)
    LOG_FILENAME = 'log.out'
    logging.basicConfig(filename=LOG_FILENAME, level=logging.DEBUG)
    logger = logging.getLogger("cidan")
    logger.debug("Program started")
    QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling,
                              True)  # enable highdpi scaling
    QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)  # use highdpi icons
    app = QApplication([])

    app.setApplicationName("cidan")
    widget = MainWindow(dev=True, preload=False)

    sys.exit(app.exec_())



import numpy as np
import pyqtgraph as pg
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure
from qtpy.QtWidgets import *


class MplCanvas(FigureCanvasQTAgg):

    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi, edgecolor=(1, 1, 1))
        fig.patch.set_facecolor((.098, .13725, .17647))
        self.axes = fig.add_subplot()
        fig.tight_layout(pad=0)
        self.axes.tick_params(colors='white')
        super(MplCanvas, self).__init__(fig)


class GraphItemLine(QWidget):
    def __init__(self, *, data, x_label, y_label, x_ticks, y_ticks,
                 display_y_axis_ticks=True, display_axis_labels=True, aspect_ratio_x=9,
                 display_roi_labels=False, roi_labels=[],
                 aspect_ratio_y=1, colors=False):
        super(GraphItemLine, self).__init__()

        self.setStyleSheet("""
                                QLabel {
                                background-color: rgba(0,0,0,0%)
                                }""")
        layout = QVBoxLayout()
        layout_h = QHBoxLayout()

        self.data = data
        self.y_label = y_label
        # if len(data.shape) ==1:
        # data = data.reshape((-1, 1))
        # data = data.transpose((1,0))
        self.graph = pg.PlotWidget()

        self.graph.showGrid(x=True, y=True, alpha=0.3)
        if (type(data) == list):
            if type(colors) == list:
                for x, color in zip(data, colors):
                    pen = pg.mkPen(color=color, width=2)
                    if type(x) != np.ndarray:
                        print("Please recalculate time traces")
                    else:
                        self.graph.plot(x, pen=pen)
            else:
                for x in data:
                    self.graph.plot(x)
            if (display_roi_labels):
                for num, name in enumerate(roi_labels):
                    text_box = pg.TextItem(y_label + "_" + str(name), anchor=(0, .5),
                                           fill=pg.mkBrush(color=(0, 0, 0)))

                    self.graph.addItem(text_box)
                    text_box.setPos(-5, num)
        else:
            self.graph.plot(data)
        self.graph.getPlotItem().getViewBox().setMouseEnabled(True, False)
        self.graph.getPlotItem().setLabel("left", y_label)
        self.graph.getPlotItem().setLabel("bottom", x_label)
        # self.graph.axes.xticks(ticks=list(range(data.shape[0])), labels=x_ticks)
        # if display_y_axis_ticks and y_ticks != None:
        #     self.graph.axes.yaxis.set_ticks(ticks=list(range(data.shape[0])))
        #     self.graph.axes.set_yticklabels(labels=y_ticks)
        # if not display_y_axis_ticks:
        #     self.graph.axes.yaxis.set_ticks([])

        # layout_h.addWidget(QLabel(y_label))
        # test = QLabel("")
        # test.setMinimumHeight(200)
        # layout_h.addWidget(test)
        layout_h.addWidget(self.graph)

        layout.addLayout(layout_h, stretch=10)
        # layout.addWidget(QLabel(x_label), alignment=QtCore.Qt.AlignCenter,stretch=1)

        self.setLayout(layout)
        # self.setMinimumHeight(200)
        layout.setContentsMargins(5, 5, 5, 5)
        layout_h.setContentsMargins(0, 0, 00, 0)



from setuptools import setup, find_packages


try:
    with open("README.md", 'r') as f:
        long_description = f.read()

except:
    long_description = ""


setup(
    name='cidan',
    version='0.1.37',
    description='cidan-Calcium Imaging Data ANalysis',
    license="MIT",
    long_description=long_description,

    long_description_content_type="text/markdown",
    author='Sam Schickler',
    author_email='sschickl@ucsd.edu',
    # url="http://www.foopackage.com/",
    packages=find_packages(),
    install_requires=["numpy", "QtPy", "QDarkStyle", "pybind11", "pyqtgraph==0.11.0rc0",
                      "pyside2",
                      "dask[complete]", "matplotlib", "scipy", "tiffile",
                      "scikit-image", "pybind11", "hnswlib", "pillow",
                      'tifffile', "zarr", "neurofinder", "scikit-learn", 'pandas', 'future',
                      "peakutils", "requests", "h5py"],
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'cidan = cidan.__main__:main',
        ]
    },
)



import logging
from functools import reduce

import numpy as np
from PIL import Image
from dask import delayed
from skimage import feature

from cidan.LSSC.SpatialBox import SpatialBox
from cidan.LSSC.functions.data_manipulation import pixel_num_to_2d_cord
from cidan.LSSC.functions.eigen import loadEigenVectors
from cidan.LSSC.functions.pickle_funcs import *
from cidan.LSSC.functions.roi_extraction import roi_extract_image, combine_rois
from cidan.LSSC.functions.roi_filter import filterRoiList
from cidan.LSSC.process_data import process_data

logger1 = logging.getLogger("cidan.DataHandler")


def calculate_roi_extraction(self, progress_signal=None):
    """
    Extracts Rois and sets them to self.rois
    """
    if self.global_params["need_recalc_eigen_params"] or self.global_params[
        "need_recalc_roi_extraction_params"] or self.global_params[
        "need_recalc_box_params"] or self.global_params[
        "need_recalc_dataset_params"] or \
            self.global_params["need_recalc_filter_params"]:
        assert (int(
            self.box_params[
                "total_num_spatial_boxes"] ** .5)) ** 2 == self.box_params[
                   "total_num_spatial_boxes"], "Please make sure Number of Spatial Boxes is a square number"
        try:
            self.calculate_filters(progress_signal=progress_signal)
            eigen_need_recalc = self.global_params["need_recalc_eigen_params"] or \
                                self.global_params[
                                    "need_recalc_box_params"]
            self.global_params["need_recalc_eigen_params"] = False
            self.global_params[
                "need_recalc_roi_extraction_params"] = False
            temp_params = self.box_params.copy()
            self.global_params[
                "need_recalc_box_params"] = False
            image_data = []
            for num, trial_num in enumerate(self._trials_loaded_indices):
                if type(self.dataset_trials_filtered_loaded[num]) == type(delayed(min)()):
                    image_data.append(self.compute_trial_filter_step(
                            trial_num,
                            loaded_num=num, save_data=False))
                else:
                    image_data.append(self.dataset_trials_filtered_loaded[num])
            rois = process_data(test_images=False, test_output_dir="",
                                save_dir=self.save_dir_path,
                                shape=self.shape,
                                save_intermediate_steps=self.global_params[
                                    "save_intermediate_steps"],
                                image_data_filtered=self.dataset_trials_filtered_loaded,
                                image_data=self.dataset_list,
                                crop=[self.dataset_params["crop_x"],
                                      self.dataset_params["crop_y"]] if
                                self.dataset_params["crop_stack"] else False,
                                slicing=[self.dataset_params["slice_start"][0],
                                         self.dataset_params["slice_every"][0]] if
                                self.dataset_params["slice_stack"] else [0, 1],

                                eigen_vectors_already_generated=(not
                                                                      eigen_need_recalc) and
                                                                     self.global_params[
                                                                         "save_intermediate_steps"] and self.eigen_vectors_exist,
                                save_embedding_images=True,
                                total_num_time_steps=self.box_params[
                                         "total_num_time_steps"],
                                total_num_spatial_boxes=self.box_params[
                                         "total_num_spatial_boxes"],
                                spatial_overlap=self.box_params[
                                                         "spatial_overlap"] // 2,

                                metric=self.eigen_params["metric"],
                                knn=self.eigen_params["knn"],
                                accuracy=self.eigen_params["accuracy"],
                                eigen_accuracy=self.eigen_params[
                                         "eigen_accuracy"],
                                connections=self.eigen_params[
                                         "connections"],
                                normalize_w_k=self.eigen_params[
                                         "normalize_w_k"],
                                num_eig=self.eigen_params["num_eig"],
                                merge=self.roi_extraction_params["merge"],
                                num_rois=self.roi_extraction_params[
                                         "num_rois"],
                                refinement=self.roi_extraction_params[
                                         "refinement"],
                                num_eigen_vector_select=
                                     self.roi_extraction_params[
                                         "num_eigen_vector_select"],
                                max_iter=self.roi_extraction_params[
                                         "max_iter"],
                                roi_size_min=self.roi_extraction_params[
                                         "roi_size_min"],
                                fill_holes=self.roi_extraction_params[
                                         "fill_holes"],

                                elbow_threshold_method=
                                     self.roi_extraction_params[
                                         "elbow_threshold_method"],
                                elbow_threshold_value=
                                     self.roi_extraction_params[
                                         "elbow_threshold_value"],
                                eigen_threshold_method=
                                     self.roi_extraction_params[
                                         "eigen_threshold_method"],
                                eigen_threshold_value=
                                     self.roi_extraction_params[
                                         "eigen_threshold_value"],
                                merge_temporal_coef=
                                     self.roi_extraction_params[
                                         "merge_temporal_coef"],
                                roi_size_max=self.roi_extraction_params[
                                    "roi_size_max"],
                                pca=self.filter_params["pca"],
                                pca_data=self.pca_decomp if self.filter_params[
                                    "pca"] else False,
                                roi_eccentricity_limit=
                                self.roi_extraction_params[
                                    "roi_eccentricity_limit"],
                                local_max_method=
                                self.roi_extraction_params[
                                    "local_max_method"],
                                area_stop_threshold=self.roi_extraction_params[
                                    "area_stop_threshold"],
                                progress_signal=progress_signal,
                                widefield=self.widefield,
                                image_data_mask=self.image_data_mask_flat.reshape(
                                    self.shape) if self.widefield else None)
            self.box_params_processed = temp_params
            self.delete_roi_vars()
            self.save_new_param_json()
            roi_valid_list = filterRoiList(rois, self.shape)

            rois = [x for x, y in zip(rois, roi_valid_list) if
                    y >= self.roi_extraction_params[
                        "roi_circ_threshold"]]
            self.rois_dict = {(num + 1): {"pixels": roi, "index": num} for num, roi in
                              enumerate(rois)}
            self.save_rois(self.rois)
            print("Calculating Time Traces")
            self.gen_roi_display_variables()
            # self.classes["0"]["rois"] = self.rois
            self.calculate_time_traces()
            """import json
with open("/Users/sschickler/Code_Devel/LSSC-python/roi_list.json", "r") as f:
    # test2=f.read()
    test =json.loads(f.read())
    rois = [[(y[0]-self.dataset_params["crop_x"][0],y[1]-self.dataset_params["crop_y"][0]) for y in x["coordinates"]]for x in test]
    from cidan.LSSC.functions.data_manipulation import cord_2d_to_pixel_num
test234 = [cord_2d_to_pixel_num(np.array(roi).transpose(), self.shape) for roi in rois]"""

            self.rois_loaded = True
        except FileNotFoundError as e:
            self.global_params["need_recalc_eigen_params"] = False
            logger1.error(e)
            print(
                "Please try again there was an internal error in the roi extraction process")
            raise AssertionError()
def gen_roi_display_variables(self):
    """
    Generates the other variables associated with displaying ROIs
    """
    self.roi_circ_list = filterRoiList(self.rois, self.shape,
                                       self.roi_extraction_params[
                                           "roi_circ_threshold"])
    roi_list_2d_cord = [
        pixel_num_to_2d_cord(x, volume_shape=self.shape) for x in
        self.rois]
    self.roi_max_cord_list = [np.max(x, axis=1) for x in roi_list_2d_cord]
    self.roi_min_cord_list = [np.min(x, axis=1) for x in roi_list_2d_cord]
    self.pixel_with_rois_flat = np.zeros(
        [self.shape[0] * self.shape[1]])
    self.pixel_with_rois_sep_flat = np.zeros(
        [len(self.rois), self.shape[0] * self.shape[1]])
    self.pixel_with_rois_color_flat = np.zeros(
        [self.shape[0] * self.shape[1], 3])
    edge_roi_image = np.zeros([self.shape[0], self.shape[1]])
    for index, roi_key in enumerate(self.rois_dict.keys()):
        num = roi_key
        roi = self.rois_dict[roi_key]["pixels"]
        cur_color = self.color_list[num % len(self.color_list)]
        self.pixel_with_rois_flat[roi] = num
        self.pixel_with_rois_sep_flat[index, roi] = num
        self.pixel_with_rois_color_flat[roi] = cur_color
        roi_edge = np.zeros(
            [self.shape[0] * self.shape[1]])
        roi_edge[roi] = 255
        edge_roi_image += feature.canny(np.reshape(roi_edge,
                                                   [self.shape[0],
                                                    self.shape[1]]))

    edge_roi_image[edge_roi_image > 255] = 255
    self.edge_roi_image_flat = np.reshape(edge_roi_image, [-1, 1]) * 255
    self.pixel_with_rois_color = np.reshape(self.pixel_with_rois_color_flat,
                                            [self.shape[0],
                                             self.shape[1], 3])
    # self.all_roi_contours = []
    # pixel_with_rois = self.pixel_with_rois_flat.reshape([self.shape[0],self.shape[1]])
    # for num in range(len(self.rois)):
    #
    #
    #     image_temp = pixel_with_rois==num+1
    #
    #     # edge = feature.canny(
    #     #     np.sum(image_temp, axis=2) / np.max(np.sum(image_temp, axis=2)))
    #     # image[edge] = 1
    #     # image_temp = ndimage.morphology.binary_dilation(image_temp)
    #     test = measure.label(image_temp, background=0, connectivity=1)
    #     # image_temp = ndimage.morphology.binary_erosion(image_temp)
    #     #
    #     # image_temp = ndimage.morphology.binary_erosion(image_temp)
    #     # image_temp = ndimage.binary_closing(image_temp)
    #     # print(test.max())
    #     for x in range(test.max()):
    #         image = np.zeros((self.shape[0], self.shape[1]), dtype=float)
    #         image[test == x + 1] = 1
    #         contour = find_contours(image, .3)
    #         self.all_roi_contours.append(contour)
    try:
        self.eigen_norm_image = np.asarray(Image.open(
            os.path.join(self.save_dir_path,
                         "embedding_norm_images/embedding_norm_image.png")))
    except:
        print("Can't generate eigen Norm image please try again")
    self.gen_class_display_variables()


def gen_class_display_variables(self):
    roi_keys = set(self.rois_dict.keys())

    keys = self.classes.keys()
    for key in [key for key in keys if key != "Unassigned"]:
        for roi_key in self.classes[key]["rois"]:
            if roi_key in roi_keys:
                roi_keys.remove(roi_key)
    self.classes["Unassigned"]["rois"] = list(roi_keys)
    roi_list_1d_cord_by_class = [[
        (self.rois_dict[x]["index"], self.rois_dict[x]["pixels"]) for x in
        self.classes[key]["rois"]] for key in keys]
    pixel_with_rois_class_flat_construct = np.zeros(

        [len(self.rois_dict), self.shape[0] * self.shape[1], 3])

    for class_index, rois in zip(keys, roi_list_1d_cord_by_class):
        cur_color = self.classes[class_index]["color"]
        for num, roi in rois:
            pixel_with_rois_class_flat_construct[num, roi] = cur_color
    num_rois_per_pixel = np.sum(
        0 != (np.sum(pixel_with_rois_class_flat_construct, axis=2)), axis=0)
    num_rois_per_pixel[num_rois_per_pixel == 0] = 1
    self.pixel_with_rois_class_flat = np.sum(pixel_with_rois_class_flat_construct,
                                             axis=0) / num_rois_per_pixel.reshape(-1, 1)

    self.pixel_with_rois_class = np.reshape(self.pixel_with_rois_class_flat,
                                            [self.shape[0],
                                             self.shape[1], 3])

def genRoiFromPoint(self, point, growth_factor=1.0):
    """
    Generates an roi from an initial point in the image
    Parameters
    ----------
    point : List[int, int]
        2d cordinates of a point in the image
    growth_factor : float
        how much to grow the roi

    Returns
    -------
    the pixels in said roi in there 1d form
    """
    if self.widefield:
        print("this function isn't available for widefield data")
        return []
    spatial_boxes = [SpatialBox(box_num=x,
                                total_boxes=self.box_params_processed[
                                    "total_num_spatial_boxes"],
                                image_shape=self.shape,
                                spatial_overlap=self.box_params_processed[
                                                    "spatial_overlap"] // 2) for x
                     in
                     range(self.box_params_processed["total_num_spatial_boxes"])]
    boxes_point_in = [x for x in spatial_boxes if x.pointInBox(point)]

    rois = []
    for box in boxes_point_in:
        all_eigen_vectors_list = []
        for temporal_box_num in range(
                self.box_params_processed["total_num_time_steps"]):
            all_eigen_vectors_list.append(
                loadEigenVectors(spatial_box_num=box.box_num,
                                 time_box_num=temporal_box_num,
                                 save_dir=self.save_dir_path).compute())

        all_eigen_vectors = np.hstack(all_eigen_vectors_list)
        box_point = box.point_to_box_point(point)
        box_point_1d = box_point[0] * box.shape[1] + box_point[1]
        roi = roi_extract_image(e_vectors=all_eigen_vectors,
                                original_shape=box.shape, original_2d_vol=None,
                                merge=False, box_num=box.box_num,
                                num_rois=1, refinement=True,
                                num_eigen_vector_select=
                                self.roi_extraction_params[
                                    "num_eigen_vector_select"],
                                max_iter=1,
                                roi_size_min=0,
                                fill_holes=self.roi_extraction_params[
                                    "fill_holes"],

                                elbow_threshold_method=True,
                                elbow_threshold_value=growth_factor,
                                eigen_threshold_method=
                                self.roi_extraction_params[
                                    "eigen_threshold_method"],
                                eigen_threshold_value=
                                self.roi_extraction_params[
                                    "eigen_threshold_value"],
                                merge_temporal_coef=
                                self.roi_extraction_params[
                                    "merge_temporal_coef"],
                                roi_size_limit=1000,
                                initial_pixel=box_point_1d,
                                print_info=False,
                                roi_eccentricity_limit=1).compute()

        if len(roi) > 0 and box_point_1d in roi[0]:
            rois.append(box.redefine_spatial_cord_1d(roi).compute()[0])
    if len(rois) > 0:
        final_roi = reduce(combine_rois, rois)
    else:
        return []

    return final_roi



import json
import logging
import os
import sys

import qdarkstyle
from PySide2.QtWidgets import QMessageBox
from qtpy import QtWidgets

from cidan.GUI.Data_Interaction.DataHandler import DataHandler
from cidan.GUI.Data_Interaction.Signals import StrIntSignal
from cidan.GUI.Data_Interaction.Thread import Thread
from cidan.GUI.Inputs.FileInput import createFileDialog

logger = logging.getLogger("cidan.GUI.Data_Interaction.PreprocessThread")


class OpenDatasetThread(Thread):

    def __init__(self, main_widget):
        super().__init__(main_widget)
        self.reportProgress = StrIntSignal()
        self.reportProgress.sig.connect(main_widget.console.updateProgressBar)
        self.signal.sig.connect(lambda x: self.endThread(x))

    def run(self):
        if self.main_widget.dev:
            self.main_widget.data_handler = DataHandler(data_path=self.data_path,
                                                        trials=self.trials,
                                                        save_dir_path=self.save_dir_path,
                                                        save_dir_already_created=self.save_dir_already_created,
                                                        load_into_mem=self.load_into_mem,
                                                        auto_crop=self.auto_crop,
                                                        mask_path=self.mask_path,
                                                        widefield=self.main_widget.widefield)
            # self.main_widget.data_handler.calculate_filters(self.reportProgress,
            #                                                 auto_crop=self.auto_crop)

            self.signal.sig.emit(True)
        else:
            try:
                self.main_widget.data_handler = DataHandler(data_path=self.data_path,
                                                            trials=self.trials,
                                                            save_dir_path=self.save_dir_path,
                                                            save_dir_already_created=self.save_dir_already_created,
                                                            load_into_mem=self.load_into_mem,
                                                            auto_crop=self.auto_crop,
                                                            mask_path=self.mask_path,
                                                            widefield=self.main_widget.widefield)
                # self.main_widget.data_handler.calculate_filters(self.reportProgress,
                #                                                 auto_crop=auto_cropself.auto_crop)
                self.signal.sig.emit(True)
            except Exception as e:
                logger.error(e)
                print("Unexpected error:", sys.exc_info()[0])
                self.main_widget.console.updateText("Unexpected error: " +
                                                    sys.exc_info()[0])
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage("Unexpected error: " + str(e))
                self.signal.sig.emit(False)

    def runThread(self, data_path, trials, save_dir_path, save_dir_already_created,
                  load_into_mem, widefield=False, mask_path=None):
        self.data_path = data_path
        self.trials = trials
        self.save_dir_path = save_dir_path
        self.save_dir_already_created = save_dir_already_created
        self.load_into_mem = load_into_mem
        self.auto_crop = False
        self.widefield = widefield
        self.mask_path = mask_path

        if not any([x.isRunning() for x in self.main_widget.thread_list]):
            print("Opening Dataset")
            self.main_widget.console.updateText("Opening Dataset")

            # self.button.setEnabled(False)
            if not save_dir_already_created and self.load_into_mem and len(
                    trials) == 1 and not widefield:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Information)
                msg.setStyleSheet(qdarkstyle.load_stylesheet())

                msg.setText(
                    "Do you want auto crop out motion correction artifacts(recommended)?")
                # msg.setInformativeText("This is additional information")
                # msg.setWindowTitle("MessageBox demo")
                # msg.setDetailedText("The details are as follows:")
                msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
                retval = msg.exec_()
                if retval == 16384:
                    self.auto_crop = True
            #load json at save_dir_path+parameters.json
            try:
                if save_dir_already_created and not params["dataset_params"]["single_file_mode"] and not os.path.isdir(params["dataset_params"]["dataset_folder_path"]):
                    file = createFileDialog(directory="~/Desktop", forOpen=True,
                                            isFolder=False, name="Couldn't Find dataset, please select the dataset folder")
                    self.data_path = os.path.dirname(file)
                self.start()

            except Exception as e:
                logger.error(e)
                print("Unexpected error:", sys.exc_info()[0])
                self.main_widget.console.updateText("Unexpected error: " +
                                                    sys.exc_info()[0])
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage("Unexpected error: " + str(e))


        else:
            print(
                "Previous process in process, please wait to start new one till "
                "finished")

    def endThread(self, success):
        self.main_widget.init_w_data()



from qtpy import QtCore
from qtpy.QtGui import *
from qtpy.QtWidgets import *

from cidan.GUI.ListWidgets.ROIItemModule import ROIItemModule
from cidan.GUI.ListWidgets.ROIItemWidget import ROIItemWidget


class ROIListModule(QFrame):
    """
    Its the roi list in the ROI modification tab
    """

    def __init__(self, data_handler, roi_tab, select_multiple=False, display_time=True,
                 class_version=False):
        super().__init__()
        self.current_selected_roi = None
        self.select_multiple = select_multiple
        self.display_time = display_time
        self.roi_tab = roi_tab
        self.class_version = class_version
        self.color_list = data_handler.color_list
        self.last_selected_roi = 0
        self.list = QListView()


        self.setStyleSheet("QListView::item { border-bottom: 1px solid rgb(50, 65, " +
                           "75); } " + """QListView::item::pressed,
 {
  background-color: #19232D;
  border: 1px solid #32414B;
  color: #F0F0F0;
  gridline-color: #32414B;
  border-radius: 4px;
}""")
        self.top_labels_layout = QHBoxLayout()
        self.top_labels_layout.setContentsMargins(0, 0, 0, 0)
        label1 = QLabel(text="ROI Selected")
        label1.setMaximumWidth(170 * ((self.logicalDpiX() / 96.0)))
        self.top_labels_layout.addWidget(label1, alignment=QtCore.Qt.AlignRight)
        label2 = QLabel(text="ROI Num")
        label2.setMaximumWidth(140 * ((self.logicalDpiX() / 96.0)))
        self.top_labels_layout.addWidget(label2, alignment=QtCore.Qt.AlignLeft)
        self.num_roi_selected_label = QLabel(text="")
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)
        self.top_labels_layout.addWidget(self.num_roi_selected_label, alignment=QtCore.Qt.AlignRight)
        self.left_arrow = QToolButton()
        self.left_arrow.setArrowType(QtCore.Qt.LeftArrow)

        self.right_arrow = QToolButton()
        self.right_arrow.setArrowType(QtCore.Qt.RightArrow)
        self.left_arrow.clicked.connect(lambda x: self.select_roi_next(False))
        self.right_arrow.clicked.connect(lambda x: self.select_roi_next(True))
        self.left_arrow.setStyleSheet("QWidget {border: 0px solid #32414B;}")
        self.right_arrow.setStyleSheet("QWidget {border: 0px solid #32414B;}")
        self.top_labels_layout.addWidget(self.left_arrow,
                                         alignment=QtCore.Qt.AlignRight)
        self.top_labels_layout.addWidget(self.right_arrow,
                                         alignment=QtCore.Qt.AlignRight)
        if display_time:
            label3 = QLabel(text="Time Trace On")
            label3.setMaximumWidth(100 * ((self.logicalDpiX() / 96.0 - 1) / 2 + 1))
            self.top_labels_layout.addWidget(label3, alignment=QtCore.Qt.AlignRight)
        self.setMinimumHeight(200 * ((self.logicalDpiX() / 96.0 - 1) / 2 + 1))
        self.model = QStandardItemModel(self.list)
        self.list.setModel(self.model)
        self.layout = QVBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addLayout(self.top_labels_layout)
        self.layout.addWidget(self.list)
        self.roi_item_list = []
        self.setLayout(self.layout)

    def keyPressEvent(self, event):
        self.roi_tab.keyPressEvent(event)

    def select_roi_next(self, next=True):
        if next:
            if len(self.roi_tab.data_handler.rois) > self.last_selected_roi + 1:
                self.set_current_select(self.last_selected_roi + 1, force_on=True,
                                        input_key=False)
        else:
            if 0 <= self.last_selected_roi - 1:
                self.set_current_select(self.last_selected_roi - 1, force_on=True,
                                        input_key=False)

    def set_current_select(self, num, force_on=False, input_key=True):
        if input_key:
            num = self.roi_tab.data_handler.rois_dict[num]["index"]
        self.last_selected_roi = num
        self.list.setCurrentIndex(self.model.index(int(num), 0))
        self.roi_time_check_list[num] = not self.roi_time_check_list[num]
        self.roi_item_list[num].select_check_box(force_on=force_on)
        # self.roi_module_list[num-1].
        if self.display_time:
            self.roi_item_list[num].select_time_check_box()
    def update_select_number(self):
        if self.select_multiple:
            num_selected = len(self.currently_selected_rois_list)
            self.num_roi_selected_label.setText("Num ROIs Selected: {}  ".format(num_selected))
        else:
            self.num_roi_selected_label.setText("")
    @property
    def currently_selected_rois_list(self):
        return [i for i, j in enumerate(self.roi_item_list) if j.check_box.checkState()]
    def set_list_items(self, rois_dict):
        self.rois_dict = rois_dict
        for x in range(self.model.rowCount()):
            self.model.removeRow(0)
        self.roi_item_list = []
        self.roi_module_list = []
        for num, id in enumerate(list(self.rois_dict.keys())):
            color = self.color_list[
                id % len(self.color_list)] if not self.class_version else \
            self.roi_tab.data_handler.classes[[key for key in
                                               self.roi_tab.data_handler.classes.keys()
                                               if id in
                                               self.roi_tab.data_handler.classes[key][
                                                   "rois"]][0]]["color"]
            item = ROIItemWidget(self.roi_tab,
                                 color, self,
                                 id=id, roi_num=num, display_time=self.display_time
                                 )
            item1 = ROIItemModule(color, id=id, num=num,
                                  roi_tab=self.roi_tab)
            item1.setSelectable(False)
            self.roi_item_list.append(item)
            self.model.appendRow(item1)
            self.roi_module_list.append(item1)
            self.list.setIndexWidget(item1.index(), item)
        self.roi_time_check_list = [False] * len(self.roi_item_list)
        if hasattr(self.roi_tab, "tab_selector_roi"):
            self.roi_tab.tab_selector_roi.setCurrentIndex(1)
        # self.roi_item_list[0].select_check_box()
        # self.roi_item_list[0].select_time_check_box()
    # def change(self):
    #     # This is a way of running the select roi function when a checkbox is clicked there
    #     # needed to be a work around because can't just connect a signal to it
    #     for num, item, check_val in zip(range(1,len(self.roi_time_check_list)+1),self.roi_item_list,self.roi_time_check_list):
    #         if item.checkState() != check_val:
    #             self.roi_time_check_list[num-1] = item.checkState()
    #             if item.checkState():
    #                 self.roi_tab.selectRoi(num)
    #             else:
    #                 self.roi_tab.deselectRoi(num)
